#!/usr/bin/env python
# -*- coding: utf-8 -*-
# @Project      : AI.  @by PyCharm
# @File         : chat_deeplx
# @Time         : 2024/4/23 18:16
# @Author       : betterme
# @WeChat       : meutils
# @Software     : PyCharm
# @Description  :

from meutils.pipe import *
from meutils.notice.feishu import send_message
from meutils.serving.fastapi.dependencies.auth import get_bearer_token, HTTPAuthorizationCredentials

from fastapi import APIRouter, File, UploadFile, Query, Form, Depends, Request, HTTPException, status

from openai.types.chat import ChatCompletion, ChatCompletionChunk

from chatllm.completions.cf import Completions

from chatllm.schemas.openai_api_protocol import ChatCompletionRequest, UsageInfo
from chatllm.utils.openai_utils import to_openai_completion_params, openai_response2sse

router = APIRouter()

ChatCompletionResponse = Union[ChatCompletion, List[ChatCompletionChunk]]

send_message = lru_cache(send_message)


@router.post("/chat/completions")
async def create_chat_completions(
    request: ChatCompletionRequest,
    auth: Optional[HTTPAuthorizationCredentials] = Depends(get_bearer_token),
):
    logger.debug(request)

    api_key = auth and auth.credentials or None
    if api_key is None:
        detail = {
            "error": {
                "message": "",
                "type": "invalid_request_error",
                "param": None,
                "code": "invalid_api_key",
            }
        }
        raise HTTPException(status_code=status.HTTP_401_UNAUTHORIZED, detail=detail)

    api_key = np.random.choice(api_key.strip().split(','))  # 轮询多个 api_key

    model_map = {
        "dreamshaper-8-lcm": "@cf/lykon/dreamshaper-8-lcm",
        "stable-diffusion-v1-5-img2img": "@cf/runwayml/stable-diffusion-v1-5-img2img",
        "stable-diffusion-v1-5-inpainting": "@cf/runwayml/stable-diffusion-v1-5-inpainting",
        "stable-diffusion-xl-base-1.0": "@cf/stabilityai/stable-diffusion-xl-base-1.0",
        "stable-diffusion-xl-lightning": "@cf/bytedance/stable-diffusion-xl-lightning",
        "*": "@cf/bytedance/stable-diffusion-xl-lightning",
    }
    request.model = model_map.get(request.model, model_map["*"])  # 最快
    response = Completions(api_key).acreate(request)
    return openai_response2sse(response)


@router.post("/audio/transcriptions")
async def create_audio_transcriptions(
    file: Union[UploadFile] = File(...),  # 文件链接
    auth: Optional[HTTPAuthorizationCredentials] = Depends(get_bearer_token),
):
    # logger.debug(request)

    api_key = auth and auth.credentials or None
    if api_key is None:
        detail = {
            "error": {
                "message": "",
                "type": "invalid_request_error",
                "param": None,
                "code": "invalid_api_key",
            }
        }
        raise HTTPException(status_code=status.HTTP_401_UNAUTHORIZED, detail=detail)

    api_key = np.random.choice(api_key.strip().split(','))  # 轮询多个 api_key

    _ = await Completions(api_key).acreate_asr(file)

    return _


if __name__ == '__main__':
    from meutils.serving.fastapi import App

    app = App()

    app.include_router(router, '/v1')

    app.run()
    # for i in range(10):
    #     send_message(f"兜底模型", title="github_copilot")



#!/usr/bin/env python
# -*- coding: utf-8 -*-
# @Project      : AI.  @by PyCharm
# @File         : prompt_templates
# @Time         : 2023/8/6 15:41
# @Author       : betterme
# @WeChat       : meutils
# @Software     : PyCharm
# @Description  :

from langchain.prompts import SystemMessagePromptTemplate, HumanMessagePromptTemplate, ChatPromptTemplate

context_prompt_template = """
根据提供的信息，以简洁、专业的方式回答用户的问题。如果无法提供答案，请回复：“根据提供的信息无法回答该问题”或“没有提供足够的信息”。请不要编造信息，答案必须使用中文。

已知信息：
```
{context}
```

问题：
{question}

让我们逐步思考并给出答案：
""".strip()  # Let's think step by step

summary_prompt_template = """
请你充当“文本摘要模型”，要求：
1. 能够从文本中提取关键信息，抓住内容本质。
2. 生成准确、没有个人意见或偏见的公正连贯的摘要。
3. 在生成的摘要中尽量减少重复和冗余，在保留基本信息的同时保持可读性与连贯性。

输入文本：{text}
Let's think step by step, 根据输入文本生成一个清晰简洁的摘要：
""".strip()

question_generation_prompt_template = """
请扮演一个“问题生成模型”，要求：
1. 理解语义上下文、预测用户意图、生成清晰、有针对性的问题
2. 从给定的信息或文本中提取关键信息，理解语境，然后根据需要产生有启发性和相关性的问题。
3. 你的目标是产生与输入文本相关且有用的问题，以帮助用户进一步思考和探索。

输入文本：{text}
Let's think step by step, 根据输入文本生成最相关的5个问题：
""".strip()

"""
请扮演阅读理解模型。
1. 描述角色的特征：应具有良好的理解能力、快速而准确地分析和回答问题的能力，同时保持客观并有效地提供相关信息。
2. 必备技能：熟练掌握阅读和语言理解技巧，具备广泛的知识储备和信息检索能力，能够理解不同类型的文本并提供准确的答案。
3. 典型活动示例：阅读和理解文章、故事、新闻报道、指南和其他各种文本形式，从中提取关键信息，回答相关问题，并进行综合分析和总结。
4. 目标设定：提供准确、全面且有意义的回答，为用户提供最佳的阅读理解经验，帮助他们更好地理解和拓展知识。
5. 合理且清晰简洁的提示，使用明确的语言描述要求。

请马上以阅读理解模型的身份开始。
""".strip()

# https://mp.weixin.qq.com/s/rtdTnlrZHuHjB1paUNTssQ
system_template = """
你将会得到一个由三个引号分隔的文档内容和一个问题，请使用三个引号内的内容，简洁、专业地回答用户的问题。
如果无法得到答案，请回复：“根据已知信息无法回答该问题”或“没有提供足够的信息”。请勿编造信息，答案必须使用中文。
""".strip()

messages = [
    SystemMessagePromptTemplate.from_template(system_template),
    HumanMessagePromptTemplate.from_template('"""{context}"""\n问题：{question}'),
]
CHAT_CONTEXT_PROMPT = ChatPromptTemplate.from_messages(messages)

system_template = """
你将会得到一个由三个引号分隔的文档内容和一个问题。
你的任务是只使用提供的文档内容来回答问题，并引用“用于回答问题的文档内容段落”。
如果文档内容中没有包含用于回答该问题所需的信息，则简单地返回：“信息不足”。
如果文档内容中提供了问题的答案，则必须使用“引文”进行注释。使用以下格式引用相关的段落。("引文": …)
""".strip()

messages = [
    SystemMessagePromptTemplate.from_template(system_template),
    HumanMessagePromptTemplate.from_template('"""{context}"""\n问题：{question}'),
]
CHAT_CONTEXT_PROMPT_WITH_SOURCE = ChatPromptTemplate.from_messages(messages)




if __name__ == '__main__':
    from meutils.pipe import *
    from langchain_community.chat_models import ChatOpenAI
    from langchain.chains import LLMChain

    llm = ChatOpenAI()
    prompt = CHAT_CONTEXT_PROMPT_WITH_SOURCE

    context = """
2022年的某一天，李明开着摩托车去给客户送货，路上遇到了一只小狗，他停下车去看了一下小狗，回去开车的时候货物不见了。李明在2023年进了一批货物，货物里面居然有一只小狗。
    """

    c = LLMChain(llm=llm, prompt=prompt)
    print(c.run(context=context, question="2022年李明开摩托车遇到了什么动物？"))



#!/usr/bin/env python
# -*- coding: utf-8 -*-
# @Project      : AI.  @by PyCharm
# @File         : openai_types
# @Time         : 2023/12/19 09:46
# @Author       : betterme
# @WeChat       : meutils
# @Software     : PyCharm
# @Description  :

from meutils.pipe import *
from openai.types.chat.chat_completion import ChatCompletion
from openai.types.chat.chat_completion_chunk import ChatCompletionChunk
from openai._types import FileTypes

completion_keys = get_function_params()


class SpeechCreateRequest(BaseModel):
    input: str
    model: str = 'tts'
    voice: str = "alloy"
    response_format: Literal["mp3", "opus", "aac", "flac", "wav", "pcm"] = "mp3"
    speed: float = 1


# class ASRRequest(BaseModel):
#     file: FileTypes
#     model: str = "whisper-1"
#     language: Optional[str] = None
#     prompt: Optional[str] = None
#     response_format: Literal["json", "text", "srt", "verbose_json", "vtt"] = "json"
#     temperature: Optional[str] = None
#     timestamp_granularities: List[Literal["word", "segment"]] = None


# todo: 结构体
chat_completion = {
    "id": "chatcmpl-id",
    "object": "chat.completion",
    "created": 0,
    "model": "LLM",
    "choices": [
        {
            "message": {"role": "assistant", "content": ''},
            "index": 0,
            "finish_reason": "stop",
            "logprobs": None
        }
    ],
    "usage": {"completion_tokens": 0, "prompt_tokens": 0, "total_tokens": 0}

}

chat_completion_chunk = {
    "id": "chatcmpl-id",
    "object": "chat.completion.chunk",
    "created": 0,
    "model": "LLM",
    "choices": [
        {
            "delta": {"role": "assistant", "content": ''},
            "index": 0,
            "finish_reason": None,
            "logprobs": None
        }
    ]
}

chat_completion_chunk_stop_ = {
    "id": "chatcmpl-8pFbEr4Y0L6RjW5hIGgDwSbiD991v",
    "choices": [
        {"delta": {"content": "", "function_call": None, "role": None, "tool_calls": None},
         "finish_reason": "stop",
         "index": 0,
         "logprobs": None
         }
    ],
    "created": 1707225384,
    "model": "gpt-4-turbo",

    "object": "chat.completion.chunk",
    "system_fingerprint": "fp_04f9a1eebf"
}

# 通用
chat_completion = ChatCompletion.model_validate(chat_completion)
chat_completion_chunk = ChatCompletionChunk.model_validate(chat_completion_chunk)
chat_completion_chunk_stop = ChatCompletionChunk.model_validate(chat_completion_chunk_stop_)

# ONEAPI_SLOGAN
ONEAPI_SLOGAN = os.getenv("ONEAPI_SLOGAN", "\n\n[永远相信美好的事情即将发生.](https://api.chatllm.vip/)\n\n")

chat_completion_slogan = chat_completion.model_copy(deep=True)
chat_completion_slogan.choices[0].message.content = ONEAPI_SLOGAN

chat_completion_chunk_slogan = chat_completion_chunk.model_copy(deep=True)
chat_completion_chunk_slogan.choices[0].delta.content = ONEAPI_SLOGAN

# ERROR
chat_completion_error = chat_completion.model_copy(deep=True)
chat_completion_chunk_error = chat_completion_chunk.model_copy(deep=True)

# PPU
chat_completion_ppu = chat_completion.model_copy(deep=True)
chat_completion_ppu.choices[0].message.content = "按次收费"
chat_completion_ppu.choices[0].finish_reason = "stop"
chat_completion_chunk_ppu = chat_completion_chunk.model_copy(deep=True)
chat_completion_chunk_ppu.choices[0].delta.content = "按次收费"


class ImageRequest(BaseModel):
    prompt: str
    model: str = 'dall-e-3'
    n: int = 1
    quality: str = 'standard'
    response_format: Literal["url", "b64_json"] = "url"
    size: Literal["256x256", "512x512", "1024x1024", "1792x1024", "1024x1792"] = '1792x1024'  # 测试默认值
    style: Literal["vivid", "natural"] = "vivid"
    extra_body: dict = {}


# ChatCompletionUserMessageParam
class ChatCompletionContentTextPart(BaseModel):
    """A part of the content of a message."""
    text: str
    """The text of the content part."""
    type: Literal["text", "image", "video", "audio", "file"]
    """The type of the content part."""


class ChatCompletionContentImagePart(BaseModel):
    """A part of the content of a message."""
    text: str
    """The text of the content part."""
    type: Literal["text", "image", "video", "audio", "file"]
    """The type of the content part."""


class ChatCompletionUserMessage(BaseModel):
    role: str = "user"
    """The role of the messages author, in this case `user`."""

    # content: Union[str, List[ChatCompletionContentPart]]
    """The contents of the user message."""


class BatchRequest(BaseModel):
    completion_window: Literal["24h"]
    """The time frame within which the batch should be processed.

    Currently only `24h` is supported.
    """

    endpoint: str = "/v1/chat/completions"  # Literal["/v1/chat/completions", "/v1/embeddings", "/v1/completions"]
    """The endpoint to be used for all requests in the batch.

    Currently `/v1/chat/completions`, `/v1/embeddings`, and `/v1/completions` are
    supported. Note that `/v1/embeddings` batches are also restricted to a maximum
    of 50,000 embedding inputs across all requests in the batch.
    """

    input_file_id: str
    """The ID of an uploaded file that contains requests for the new batch.

    See [upload file](https://platform.openai.com/docs/api-reference/files/create)
    for how to upload a file.

    Your input file must be formatted as a
    [JSONL file](https://platform.openai.com/docs/api-reference/batch/requestInput),
    and must be uploaded with the purpose `batch`. The file can contain up to 50,000
    requests, and can be up to 100 MB in size.
    """

    metadata: Dict[str, str] = {}
    """Optional custom metadata for the batch."""


if __name__ == '__main__':
    # data = {"stream": True, "model": "gpt-3.5-turbo", "messages": [{"role": "user", "content": "你好"}]}
    # print(ChatCompletionRequest(request=data).request)
    # print(chat_completion_error)
    # print(chat_completion_slogan)

    # print(chat_completion_chunk_stop)
    # for i in chat_completion_error:
    #     print(i)

    pass
    print(BatchRequest(input_file_id="input_file_id",
                       endpoint="/v1/chat/completions",
                       completion_window="24h",  # 完成时间只支持 24 小时
                       metadata={
                           "description": "Sentiment classification"
                       }))



#!/usr/bin/env python
# -*- coding: utf-8 -*-
# @Project      : AI.  @by PyCharm
# @File         : kimi
# @Time         : 2023/11/29 17:00
# @Author       : betterme
# @WeChat       : meutils
# @Software     : PyCharm
# @Description  : 只要单轮对话

from meutils.pipe import *
from meutils.cache_utils import ttl_cache, diskcache
from meutils.decorators.retry import retrying
from meutils.notice.feishu import send_message
from meutils.str_utils.regular_expression import parse_url
from meutils.hash_utils import murmurhash

from chatllm.llmchain.utils import tiktoken_encoder
from chatllm.schemas.kimi.protocol import EventData
from chatllm.llmchain.completions import github_copilot

from openai.types.chat import chat_completion_chunk, chat_completion
from openai.types.chat.chat_completion_chunk import ChatCompletionChunk
from chatllm.schemas.openai_types import chat_completion_error, chat_completion_chunk_error
from chatllm.schemas.openai_types import chat_completion_chunk

send_message = partial(
    send_message,
    title="Kimi",
    url="https://open.feishu.cn/open-apis/bot/v2/hook/e2f5c8eb-4421-4a0b-88ea-e2d9441990f2"
)


class Completions(object):
    def __init__(self, **client_params):
        self.api_key = client_params.get('api_key')  # state_file
        self.access_token = self.get_access_token(self.api_key)

    def create(
        self,
        messages: Union[str, List[Dict[str, Any]]],
        **kwargs,
    ):
        if isinstance(messages, str):
            messages = [{'role': 'user', 'content': messages}]

        logger.debug(f"messages：{messages}")

        ######################################################################################################
        data = {
            "model": 'gpt-4',
            "messages": messages[-1:],  # 系统提示词怎么自定义, # Check
            "conversation_name": f"{murmurhash(messages[:2], bins=100000)}: {messages[:2][-1]['content'][:10]}",
            **kwargs
        }  # 结构体

        # 额外参数
        if len(data.get('model')) > 20:  # 不联网
            refs = data.get('model', '').strip('kimi-').split('|')  # ['']
            refs = data.pop('refs', refs[0] and refs or [])
            logger.debug(refs)

            use_search = False if refs else data.pop('use_search', True)
            data["refs"] = refs
            data["use_search"] = use_search

        new_content = self._url2content(data["messages"][-1]["content"])
        if new_content:
            data['refs'] = []
            data['use_search'] = False
            data["messages"][-1]["content"] = new_content

        ######################################################################################################

        logger.debug(f"RequestData：{data}")

        if data.get('stream'):
            return self._stream_create(**data)
        else:
            return self._create(**data)

    def _create(self, **data):

        # todo
        # response = requests.post(url, json=json_str, headers=headers)
        # response.encoding = 'utf-8'
        # response.text.strip().split('\n\n')

        content = ''
        chunk_id = created = None
        model = data.get('model', 'kimi')
        for chunk in self._stream_create(**data):
            chunk_id = chunk.id
            created = chunk.created
            content += chunk.choices[0].delta.content

        message = chat_completion.ChatCompletionMessage(role='assistant', content=content)

        choice = chat_completion.Choice(
            index=0,
            message=message,
            finish_reason='stop',
            logprobs=None
        )

        prompt_tokens, completion_tokens = map(len, tiktoken_encoder.encode_batch([str(data.get('messages')), content]))
        total_tokens = prompt_tokens + completion_tokens

        usage = chat_completion.CompletionUsage(
            prompt_tokens=prompt_tokens,
            completion_tokens=completion_tokens,
            total_tokens=total_tokens
        )

        completion = chat_completion.ChatCompletion(
            id=chunk_id,
            choices=[choice],
            created=created,
            model=model,
            object="chat.completion",
            usage=usage

        )

        return completion

    def _url2content(self, content) -> Union[bool, str]:
        urls = parse_url(content)
        if not urls: return False

        for i, url in enumerate(urls):
            _url = f""" <url id="{i}" type="url" status="" title="" wc="">{url}</url> """
            content = content.replace(url, _url)

        return content

    def _search_plus(self, event_data: EventData):
        if event_data.event == 'search_plus':
            if event_data.msg.get("type") == "start":  # start_res
                chat_completion_chunk.choices[0].delta.content = "---\n🔎 开始搜索 🚀\n"
                yield chat_completion_chunk
            if event_data.msg.get("type") == "get_res":
                title = event_data.msg.get("title")
                url = event_data.msg.get("url")
                chat_completion_chunk.choices[0].delta.content = f"""- 🔗 [{title}]({url})\n"""
                yield chat_completion_chunk

            if event_data.msg.get("type") == "answer":
                chat_completion_chunk.choices[0].delta.content = f"""---\n\n"""
                yield chat_completion_chunk

    def _stream_create(self, **data):  # {'messages': messages, "refs": refs, "use_search": use_search}
        response = self._post(**data)
        if response is None or response.status_code != 200:
            chat_completion_chunk_error.choices[0].delta.content = hasattr(response, 'text') and response.text
            yield chat_completion_chunk_error
            return

        chunk_id = f"chatcmpl-{uuid.uuid1()}"
        created = int(time.time())
        model = data.get('model', 'kimi')
        finish_reason = None

        for chunk in response.iter_lines(chunk_size=1024):
            print(chunk.decode())

            if chunk:
                # logger.debug(chunk.decode())
                chunk = chunk.strip(b"data: ")
                if chunk.startswith(b'{"error":'):
                    logger.debug(chunk.decode())
                event_data = EventData(**json.loads(chunk))

                if event_data.event == "all_done" or event_data.error_type:
                    finish_reason = 'stop'
                    logger.info(chunk.decode())
                    # {"error_type":"validation.error","message":"参数验证错误","detail":["Content is a required field"]}
                    # {"error":{"error_type":"openai.completion.stream","message":"Kimi 现在有点累了，晚一点再来问问我吧！","detail":"runtime error: invalid memory address or nil pointer dereference"},"event":"error"}

                    # if event_data.error_type == "chat.forbidden": # 敏感内容
                    #     event_data.text = "请稍后重试！"
                    #     data['conversation_name'] = f"「XchatNew：{time.ctime()}」"
                    #     return self._stream_create(**data)

                ##############################AGI#################################
                if event_data.event == 'debug':
                    logger.debug(event_data.message)
                if event_data.event == 'search_plus':
                    logger.debug(event_data.msg)
                    yield from self._search_plus(event_data)

                ##############################AGI#################################
                if event_data.text or finish_reason:
                    # chat_completion_chunk = chat_completion_chunk.model_copy(update={"model": "kimi"}, deep=True)
                    chat_completion_chunk.model = model
                    chat_completion_chunk.id = chunk_id
                    chat_completion_chunk.created = created

                    chat_completion_chunk.choices[0].delta.content = event_data.text

                    # logger.debug(chat_completion_chunk)

                    yield chat_completion_chunk

    @retrying(predicate=lambda r: isinstance(r, str) and ("auth.token.invalid" in r))
    def _post(self, **data):
        headers = {
            'Authorization': f"Bearer {self.access_token}",
            'User-Agent': 'Mozilla/5.0 (Macintosh; Intel Mac OS X 10_15_7) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/119.0.0.0 Safari/537.36'
        }

        # {"error_type":"chat.not_found","message":"找不到指定的会话"}
        conversation_name = data.pop('conversation_name')
        chat_url = self._create_url(conversation_name=conversation_name, headers=headers)  #

        response = requests.post(
            chat_url,
            json=data,
            headers=headers,
            stream=data.get('stream'),
        )

        if response.status_code != 200:
            send_message(content=f"{response.text}\n\n{self.api_key}")
            return response.text

        return response

    @staticmethod
    def _create_url(conversation_name: Optional[str] = None, headers: Optional[dict] = None):
        headers = headers or {}
        url = "https://kimi.moonshot.cn/api/chat"
        payload = {"name": str(conversation_name), "is_example": False}
        response = requests.post(url, json=payload, headers=headers).json()
        # logger.debug(response)

        conversation_id = response.get('id')
        return f"{url}/{conversation_id}/completion/stream"

    @staticmethod
    @retrying
    def get_access_token(state_file):  # 从redis读取: 要跟会话名称一致吗【用户id区分】
        state_file = state_file or "/Users/betterme/PycharmProjects/AI/MeUtils/examples/爬虫/kimi_cookies.json"

        cookies = json.loads(Path(state_file).read_text())
        storage = cookies.get('origins', [{}])[0].get('localStorage', [{}])

        access_token = refresh_token = None
        for name2value in storage:
            if name2value.get('name') == 'access_token':
                access_token = name2value.get('value')
            if name2value.get('name') == 'refresh_token':
                refresh_token = name2value.get('value')

        return access_token


if __name__ == '__main__':
    # {"error_type":"validation.error","message":"参数验证错误","detail":["Content is a required field"]}

    data = {
        'model': 'kimi', 'messages': [{'role': 'user', 'content': '今天南京天气怎么样'}], 'stream': True,
        'use_search': True
    }
    # data = {
    #     "messages":
    #         [
    #             {"role": "user",
    #              "content": "总结一下 https://waptianqi.2345.com/wea_history/58238.htm"}
    #         ],
    #     "refs": [],
    #     "use_search": False,
    #     'stream': True
    # }



    # with timer('异步'):
    #     print([Completions().acreate(**data) for _ in range(10)] | xAsyncio)

    state_file = "/Users/betterme/PycharmProjects/AI/MeUtils/examples/爬虫/kimi_cookies.json"
    completion = Completions(api_key=state_file)

    data = {
        'model': 'kimi-clk4da83qff43om28p80|clk4da83qff43om28p80',
        'messages': [{'role': 'user', 'content': '总结一下'}],
        'stream': False,
        'use_search': False
    }

    _ = completion.create(**data)

    if isinstance(_, Generator):
        for i in tqdm(_):
            content = i.choices[0].delta.content
            print(content, end='')
    else:
        print(_)



#!/usr/bin/env python
# -*- coding: utf-8 -*-
# @Project      : AI.  @by PyCharm
# @File         : ocr
# @Time         : 2023/9/5 16:24
# @Author       : betterme
# @WeChat       : meutils
# @Software     : PyCharm
# @Description  : 通用文本图像智能分析系统

from meutils.pipe import *
from langchain.prompts import SystemMessagePromptTemplate, HumanMessagePromptTemplate, ChatPromptTemplate

# context=ocr_result, keys=question # 开票日期,开票人,收款人
ocr_ie_prompt = """
你现在的任务是从OCR文字识别的结果中提取我指定的关键信息。
OCR的文字识别结果使用```符号包围，包含所识别出来的文字，顺序在原始图片中从左至右、从上至下。我指定的关键信息使用[]符号包围。
请注意OCR的文字识别结果可能存在长句子换行被切断、不合理的分词、对应错位等问题，你需要结合上下文语义进行综合判断，以抽取准确的关键信息。
在返回结果时使用json格式，包含一个key-value对，key值为我指定的关键信息，value值为所抽取的结果。
如果认为OCR识别结果中没有关键信息key，则将value赋值为“未找到相关信息”。 请只输出json格式的结果，不要包含其它多余文字！下面正式开始：
OCR文字：```{context}```
要抽取的关键信息：[{question}]。
""".strip()

# {
#     '坐标': [
#         [358.0, 1488.0],
#         [396.0, 1488.0],
#         [396.0, 1554.0],
#         [358.0, 1554.0]
#     ],
#     '文字': '部门'
# }

ocr_qa_prompt = """
你现在的任务是根据OCR文字识别的结果回答问题。
OCR的文字识别结果使用```符号包围，包含所识别出来的文字与文字对应的坐标，顺序在原始图片中从左至右、从上至下。
请注意OCR的文字识别结果可能存在长句子换行被切断、不合理的分词、对应错位等问题，你需要结合上下文语义及坐标进行综合判断，让我们一步一步思考并准确回答问题。
OCR文字识别的结果：```{context}```
问题：{question}
""".strip()

# https://github.com/PromptExpert/Trickle-On-WeChat/tree/main
ocr_desc_prompt = """
- 你会将图片通过OCR后的文本信息整合总结，请一步一步思考，你会挖掘不同单词和信息之间的联系，
- 你会用各种信息分析方法（如：统计、聚类...等）完成信息整理任务，翻译成中文回复。
- 输出格式：
"
  # 标题
  {填充信息：通过一句话概括成标题，不超过15字}

  # 概要
  {填充信息：通过一句话描述整体内容，不超过30字}

  {填充信息：分点显示，整合信息后总结，最多不超过8点，每条信息不超过20字，保留关键值，如人名、地名...}

  # 标签
  {填充信息：为该信息3-5个分类标签，例如：#科学、#艺术、#文学、#科技}
"
""".strip()
# user_msg = "这张图是{},图中文字信息：{}".format(desc,text)



#!/usr/bin/env python
# -*- coding: utf-8 -*-
# @Project      : AI.  @by PyCharm
# @File         : multi_key_openai
# @Time         : 2024/5/22 14:16
# @Author       : betterme
# @WeChat       : meutils
# @Software     : PyCharm
# @Description  :


from meutils.pipe import *
from meutils.notice.feishu import send_message
from meutils.str_utils.regular_expression import parse_url
from meutils.db.redis_db import redis_client, redis_aclient
from meutils.config_utils.lark_utils import get_spreadsheet_values

from meutils.schemas.openai_types import chat_completion, chat_completion_chunk
from chatllm.schemas.openai_api_protocol import ChatCompletionRequest, UsageInfo
from chatllm.utils.openai_utils import openai_response2sse, to_openai_completion_params

from openai import OpenAI, AsyncOpenAI


class Completions(object):
    def __init__(self, base_url: Optional[str] = None):
        self.provider = self.base_url = base_url or 'https://api.chatfire.cn/v1'

    async def acreate(self, request: ChatCompletionRequest, **kwargs):
        # logger.debug(str(request.messages))
        data = to_openai_completion_params(request)
        completion = None
        for i in range(10):  # 轮询个数
            api_key = redis_client.lpop(self.provider)
            try:
                completion = AsyncOpenAI(
                    api_key=api_key,
                    base_url=self.base_url,
                ).chat.completions.create(**data)

                redis_client.rpush(self.provider, api_key)  # 请求成功才会，放入右侧

            except Exception as e:  # Error code: 429 - {'error': {'message': 'Your account co1fvuucp7f01qu4l34g<ak-er3hzz411h1i11gwmzti> request reached max request: 3, please try again after 1 seconds', 'type': 'rate_limit_reached_error'}}
                if all(_ not in str(e) for _ in self.fatal_errors):
                    redis_client.rpush(self.provider, api_key)  # 不包含致命错误，可放入右侧

                logger.error(e)  # todo: 错误规避措施
                send_message(f"{e}:\n{api_key}", title=self.provider)

            return completion or await completion

    @diskcache(location='api_keys_cache', ttl=5 * 60, ignore=['self'])
    def get_and_update_api_keys(self, feishu_url):  # 保活
        """ todo： check key
        定时更新&空闲更新
                1、 通过飞书更新keys
                2、检测key
                3、更新key


                调用 => key有效性检查 => 当前key `60 // rpm +1` s后，放入keys右侧 => 轮询
                """
        # 获取keys
        df = get_spreadsheet_values(feishu_url=feishu_url, to_dataframe=True)
        api_keys = list(filter(None, df[0]))

        # 判断，更新
        if redis_client.get(feishu_url) == str(api_keys):
            # logger.debug("api_keys 未更新")
            return api_keys

        redis_client.set(feishu_url, str(api_keys), ex=30 * 24 * 3600)

        redis_client.delete(self.provider)
        redis_client.rpush(self.provider, *api_keys)

        send_message(f"更新api_keys: \n{api_keys}", title=feishu_url)
        # logger.debug(api_keys)

        return api_keys

    @property
    def fatal_errors(self):
        return {'invalid_api_key', 'current state: suspended'}


if __name__ == '__main__':
    print(arun(Completions().acreate(ChatCompletionRequest(messages=[{"role": "user", "content": "你是谁"}]))))
    feishu_url = "https://xchatllm.feishu.cn/sheets/Bmjtst2f6hfMqFttbhLcdfRJnNf?sheet=79272d"
    print(Completions().get_and_update_api_keys(feishu_url))
    pass



#!/usr/bin/env python
# -*- coding: utf-8 -*-
# @Project      : AI.  @by PyCharm
# @File         : copilot
# @Time         : 2023/12/6 13:14
# @Author       : betterme
# @WeChat       : meutils
# @Software     : PyCharm
# @Description  :


import openai

from meutils.pipe import *
from meutils.cache_utils import ttl_cache
from meutils.decorators.retry import retrying
from meutils.queues.smooth_queue import SmoothQueue

from meutils.notice.feishu import send_message

from openai.types.chat.chat_completion import ChatCompletion
from openai.types.chat.chat_completion_chunk import ChatCompletionChunk

from chatllm.llmchain.completions import openai_completions
from chatllm.schemas.openai_api_protocol import ChatCompletionRequest, UsageInfo
from chatllm.utils.openai_utils import to_openai_completion_params, openai_response2sse
from chatllm.schemas.openai_types import chat_completion_chunk

from openai import OpenAI, AsyncOpenAI


class Completions(object):
    def __init__(self, **client_params):
        self.api_key = client_params.get('api_key')

        self.client = AsyncOpenAI(api_key=self.api_key)

    async def acreate(self, request: ChatCompletionRequest, **kwargs):
        data = to_openai_completion_params(request)

        start_time = time.perf_counter()

        try:
            # _ = await self.client.chat.completions.create(**data)

            async def gen():
                async for chunks in await self.client.chat.completions.create(**data):
                    logger.debug(chunks)
                    if chunks.choices[0].finish_reason == 'stop':
                        yield chunks
                        return
                    for chunk in chunks:
                        chat_completion_chunk.choices[0].delta.content = chunk.choices[0].delta.content
                        yield chat_completion_chunk

            _ = gen()

        except Exception as e:
            logger.error(e)
            send_message(f"{self.api_key}\n{e}\n耗时：{time.perf_counter() - start_time} s", "Copilot Token 异常")

            data['model'] = 'backup-gpt'
            _ = await AsyncOpenAI().chat.completions.create(**data)

        if request.stream:
            return SmoothQueue().consumer(_, delay=0.05)

        return _


if __name__ == '__main__':
    Completions().acreate(ChatCompletionRequest(model='qwen-all'))



#!/usr/bin/env python
# -*- coding: utf-8 -*-
# @Project      : AI.  @by PyCharm
# @File         : yuanbao_types
# @Time         : 2024/6/11 19:01
# @Author       : betterme
# @WeChat       : meutils
# @Software     : PyCharm
# @Description  :

from meutils.pipe import *
from meutils.ai_cv.utils import crop_image_and_upload

import orjson as json
from urllib.parse import quote, unquote

BASE_URL = "https://hunyuan.tencent.com"
API_CONV_DETAIL = "/api/conv"
API_LOGIN = "/api/oalogin"
API_GENERATE_ID = "/api/generate/id"
API_GENERATE_ID_V2 = "/api/v2/generate/id"

API_CONV_LIST = "/api/convs"
API_DELETE_CONV = "/api/conv/delete"
API_CLEAR_CONV = "/api/convs/clear"
API_GENERATE_TITLE = "/api/chat/title"
API_TITLE_MODIFY = "/api/conv/title"
API_PRE_CHAT = "/api/chat/prechat"
API_CHAT = "/api/chat"
API_REPEAT = "/api/chat/repeat"
API_STOP_CHAT = "/api/stop/chat"
API_STOP_CONVERSATION = "/api/stop/conversation"
API_COMPARE_STOP_CONVERSATION = "/api/stop/relation_convs"
API_SUITABLE = "/api/ai/suitable"
API_MODELS = "/api/models"
API_COMPLAIN = "/api/ai/complaint"
API_INFO_GENERAL = "/api/info/general"
API_COMPARE_DETAIL = "/api/relation/conv"
API_MODEL_LIST = "/api/models"
API_GENERATE_COS_KEY = "/api/resource/genUploadInfo"
API_GENERATE_PROMPTS = "/api/generate/prompts"
API_CHAT_SHARE = "/api/convs/share"
API_CHAT_QUOTA_INFO = "/api/query/chat/quotainfo"
API_CHAT_FEEDBACK = "/api/ai/feedback"
API_CHAT_GET_SHARE = "/api/convs/share"
API_MINIPROGRAM_QRCODE = "/api/weixin/getwxaqrcode"
API_SHORTCUT_CREATE = "/api/v2/shortcut/create"
API_SHORTCUT_DELETE = "/api/v2/shortcut/delete"
API_SHORTCUT_UPDATE = "/api/v2/shortcut/update"
API_SHORTCUT_LIST = "/api/v2/shortcut/list"
API_SHORTCUT_MOVE = "/api/v2/shortcut/move"
API_SHORTCUT_TAG_CREATE = "/api/v2/shortcut/tag/create"
API_SHORTCUT_TAG_DELETE = "/api/v2/shortcut/tag/delete"
API_SHORTCUT_TAG_LIST = "/api/v2/shortcut/tag/list"
API_SHORTCUT_TAG_UPDATE = "/api/v2/shortcut/tag/update"
API_SHORTCUT_USE_REPORT = "/api/v2/shortcut/use_report"
API_SHORTCUT_RECOMMEND_LIST = "/api/v2/shortcut/recent"
API_MASSAGE_CENTER_LIST = "/api/message_center/list"
API_UPDATE_MASSAGE_CENTER_STATUS = "/api/message_center/update"
API_CHAT_SHARE_NEW = "/api/conversations/share"
API_CHAT_GET_SHARE_NEW = "/api/conversations/share"
API_GET_INSPIRATION_TEMPLATE_DETAIL = "/api/inspirations/template"
API_GET_PRIVACY_STATUS = "/api/privacy/status"
API_SET_PRIVACY_STATUS = "/api/privacy/agree"
API_GET_GUIDE_INFO = "/api/get_guidance_info"
API_SET_GUIDE_INFO = "/api/update_guidance_info"
API_GET_THIRD_PLUGIN_LIST = "/api/third_plugin/list"
API_GET_FILE_COS_URL = "/api/resource/download"
API_CHAT_GET_3D_SHARE = "/api/3d/creations/share"
API_GET_FILE_COS_URL_V1 = "/api/resource/v1/download"
RTX_USER_SEARCH = "/rtx_proxy/query"
ADMIN_CHAT_API_CONV_DETAIL = "/api/admin/conv"
ADMIN_CHAT_API_CONV_LIST = "/api/admin/convs"
ADMIN_API_CHAT_SHARE = "/api/admin/share"
ADMIN_API_LOGIN = "/api/admin/oalogin"
ADMIN_ACTION_API_FEEDBACK_DASHBOARD = "/api/admin/stats/feedback"
ADMIN_ACTION_FEEDBACK_DATA_LIST = "/api/admin/feeds/list"
ADMIN_ACTION_FEEDBACK_DATA_DETAIL = "/api/admin/feeds"
ADMIN_ACTION_API_FEEDBACK_EXPORT = "/api/admin/feeds/export"
ADMIN_API_ACCOUNT_LIST = "/api/admin/account/list"
ADMIN_API_ACCOUNT_UPDATE = "/api/admin/account/update"
ADMIN_API_ACCOUNT_UPDATE_BATCH = "/api/admin/account/update_batch"
ADMIN_API_ACCOUNT_IMPORT_BATCH = "/api/admin/account/import_batch"
ADMIN_API_SHARE_LIST = "/api/admin/share/list"
ADMIN_API_SHARE_OVERVIEW = "/api/admin/share/overview"
ADMIN_KNOWLEDGE_FAQ_LIST = "/api/admin/domain/list"
ADMIN_KNOWLEDGE_FAQ_DETAIL = "/api/admin/domain"
ADMIN_KNOWLEDGE_FAQ_CHECK = "/api/admin/domain/check"
ADMIN_KNOWLEDGE_FAQ_ADD = "/api/admin/domain/add"
ADMIN_KNOWLEDGE_FAQ_MODIFY = "/api/admin/domain/modifyValue"
ADMIN_KNOWLEDGE_FAQ_DELETE = "/api/admin/domain/delete"
ADMIN_KNOWLEDGE_GRAPH_LIST = "/api/admin/graph/list"
ADMIN_KNOWLEDGE_GRAPH_DETAIL = "/api/admin/domain"
ADMIN_KNOWLEDGE_GRAPH_CHECK = "/api/admin/graph/check"
ADMIN_KNOWLEDGE_GRAPH_ADD = "/api/admin/graph/add"
ADMIN_KNOWLEDGE_GRAPH_MODIFY = "/api/admin/graph/modifyValue"
ADMIN_KNOWLEDGE_GRAPH_DELETE = "/api/admin/graph/delete"
ADMIN_ACTION_REPORT_DATA = "/api/admin/stats/overview"
ADMIN_DEPOSIT_API_WHITE_LIST = "/api/admin/deposit/white/list"
ADMIN_DEPOSIT_API_WHITE_ADD = "/api/admin/deposit/white/add"
ADMIN_DEPOSIT_API_WHITE_UPDATE = "/api/admin/deposit/white/update"
ADMIN_DEPOSIT_API_WHITE_DEL = "/api/admin/deposit/white/delete"
ADMIN_API_OPENAPI_AUDIT_LIST = "/api/admin/auditOpenAPI/list/admin"
ADMIN_API_OPENAPI_AUDIT_EXPORT = "/api/admin/auditOpenAPI/list/export"
ADMIN_API_OPENAPI_OBS_DEPT_LIST = "/api/admin/obs/dept/list"
ADMIN_API_OPENAPI_OBS_PRODUCT_LIST = "/api/admin/obs/product/list"
ADMIN_API_OPENAPI_OBS_UPDATE_STATUS = "/api/admin/auditOpenAPI"
ADMIN_API_OPENAPI_OBS_AUDIT_DETAIL = "/api/admin/auditOpenAPI"
ADMIN_API_INFO_GENERAL = "/api/admin/info/general"
TENCENT_GET_USER_INFO = "/api/getuserinfo"
TENCENT_GET_SHARE_AVATAR = "/api/convs/shareuserimage"
ADMIN_DOMAIN_APPROVE_LIST = "/api/admin/domain/approve/list"
ADMIN_DOMAIN_APPROVE_APPLY = "/api/admin/domain/approve/apply"
ADMIN_DOMAIN_APPROVE_KEYS = "/api/admin/domain/approve/keys"
ADMIN_OPERATION_CONFIGURATION = "/api/admin/operations"
ADMIN_OPERATION_CONFIGURATION_UPDATE = "/api/admin/operations/update"
ADMIN_BADCASE_LIST = "/api/admin/badcase/list"
ADMIN_BADCASE_DETAIL = "/api/admin/badcase"
ADMIN_BADCASE_ADD = "/api/admin/badcase/add"
ADMIN_BADCASE_CHECK = "/api/admin/badcase/check"
ADMIN_BADCASE_IMPORT = "/api/admin/badcase/import"
ADMIN_BADCASE_UPDATE = "/api/admin/badcase/update"
WX_JS_GET_SIGN = "/api/weixin/get_sign"
APP_AGENT_LIST = "/api/agent/v2/list"
API_USERINFO_SETPROFILEIMAGE = "/api/userinfo/setprofileimage"
API_USERINFO_SETNICKNAME = "/api/userinfo/setnickname"
API_USERINFO_SETGENDER = "/api/userinfo/setgender"
API_GET_USER_AGENT_LIST = "/api/user/agent/list"
API_CHAT_TEXT_CHECK = "/api/user/agent/conversation/textcheck"
API_CHAT_FILE_PARSE = "/api/resource/fileParse"
API_GET_VOICE_SDK_TOKEN = "/api/generate/voice_tmpkey"

API_LOGIN_OUT = "/api/login/logout"
API_UNREGISTER = "/api/unregister"
API_PHONE_LOGIN = "/api/login/phonelogin"
API_PHONE_CHANGE = "/api/changephonenumber"
API_WECHAT_CHANGE = "/api/changeweixin"
API_WECHAT_UNBIND = "/api/unbundleweixin"
GET_AGENT_INFO = "/api/user/agent/list"
GET_AGENT_CHAT_ID = "/api/generate/id"
GET_AGENT_CHAT = "/api/user/agent/v1/conversation"
GET_EXTERNAL_AGENT_CHAT = "/api/external/user/agent/conversation"
STOP_AGENT_CHAT = "/api/stop/chat"
CLEAR_AGENT_CONTEXT = "/api/stop/conversation"
REPEAT_AGENT_CHAT = "/api/chat/repeat"
AGENT_CHAT = "/api/chat"
HINT_PROMPT = "/api/user/agent/conversation/prompthint"
IMAGE_ASSETS_HISTORY = "/api/image/assets/get"
DELETE_IMAGE_ASSETS = "/api/image/assets/delete"
GET_AGENT_LIST = "/api/user/agent/list"
GET_PERSONAL_AGENT_LIST = "/api/agent/personal/agent/list"
SUITABLE_AGENT_CHAT = "/api/ai/suitable"
COMPLAIN_AGENT_CHAT = "/api/ai/complaint"
FEEDBACK_AGENT_CHAT = "/api/ai/feedback"
SHARE_AGENT_CHAT = "/api/conversations/share"
SHARE_AGENT_CHAT_V2 = "/api/conversations/v2/share"
GET_SHARE_MINI_QRCODE = "/api/weixin/getwxaqrcode"
GET_SHARE_IMAGE = "/api/aigc/v1/screenshot"
GET_AGENT_SHARE = "/api/conversations/share"
GET_DISCOVERY_CHAT = "/api/agent/conversation/discovery"
GET_AGENT_SQUARE_LIST = "/api/agent/list"
DELETE_AGENT = "/api/user/agent/delete"
UPDATE_AGENT = "/api/user/agent/update"
GET_AGENT_GUIDE = "/api/user/agent/v2/guide"
DELETE_AGENT_HISTORY = "/api/user/agent/conversation/clear"
GET_AGENT_RECOMMEND = "/api/agent/recommend"
GET_CHAT_LONGTEXT = "/api/conversation/longtext"
EXPORT_LONGTEXT = "/api/conversation/longtext/download"
GET_COS_URL = "/api/resource/v1/download"
GET_CHAT_LIST = "/api/convs"
GET_CHAT_DETAIL = "/api/conv"
GET_DISCOVERY_CARD = "/api/discovery/highlights"
GET_SHARE_PICBOOK = "/api/image/agent/ai_picture_book/asset/share"
GET_AGENT_FEEDS = "/api/agent/startFeeds"
GET_AGENT_TOOL_GUIDE = "/api/agent/toolsGuide"
GET_AGENT_GREETING_BASIC = "/api/homePageInfo"
GET_AGENT_GREETING_SPECIAL_COLUMN = "/api/user/agent/v3/guide"
GET_AGENT_GREETING_SPECIAL_COLUMN_DETAIL = "/api/user/agent/guide/detail"
GET_PROMPT_HINTS = "/api/user/agent/conversation/prompthint"


class SSEData(BaseModel):
    # data: {"type": "text"}
    # event: speech_type
    # data: status
    # event: speech_type
    # data: text
    # data: {"type": "text", "msg": "计算"}
    # data: {"type": "text", "msg": "结果为"}
    # data: {"type": "text", "msg": ":"}
    # data: {"type": "text", "msg": "\n"}
    # data: {"type": "text", "msg": "\n"}
    # data: {"type": "text", "msg": "1"}
    # data: {"type": "text", "msg": " +"}
    # data: {"type": "text", "msg": " "}
    # data: {"type": "text", "msg": "1"}
    # data: {"type": "text", "msg": " ="}
    # data: {"type": "text", "msg": " "}
    # data: {"type": "text", "msg": "2"}

    # data: {"type": "progress", "value": 0.1875}

    # data: {"type": "image",
    #        "imageUrlLow": "https://hunyuan-prod-1258344703.cos.ap-guangzhou.myqcloud.com/text2img/87c56543f333a5c9dff7cc4985dc84cb/20240611191629h0_bb05437fd6e3d380880af18cb8246dee.png?q-sign-algorithm=sha1\u0026q-ak=AKID0qSq0xJRL7h3A4nIYJFrFOJ1VlnbIm26\u0026q-sign-time=1718104589;1749640589\u0026q-key-time=1718104589;1749640589\u0026q-header-list=host\u0026q-url-param-list=\u0026q-signature=b8129ea17df63a7616f744fd466bf3ce00946601",
    #        "imageUrlHigh": "https://hunyuan-prod-1258344703.cos.ap-guangzhou.myqcloud.com/text2img/87c56543f333a5c9dff7cc4985dc84cb/20240611191629h0_bb05437fd6e3d380880af18cb8246dee.png?q-sign-algorithm=sha1\u0026q-ak=AKID0qSq0xJRL7h3A4nIYJFrFOJ1VlnbIm26\u0026q-sign-time=1718104589;1749640589\u0026q-key-time=1718104589;1749640589\u0026q-header-list=host\u0026q-url-param-list=\u0026q-signature=b8129ea17df63a7616f744fd466bf3ce00946601",
    #        "seed": 2452899368,
    #        "prompt": "画一条可爱的小狗，它有着毛茸茸的外表和充满活力的眼神，它正在一个安逸的家里玩耍，背景是舒适的沙发和视图玩具，这张画的风格是生动和活泼，构图使得小狗看起来非常可爱"}
    # data: {"type": "text",
    #        "msg": "AI生成完成，如想继续做画，请继续完整描述需求，如：请帮我生成一张图片：轻舟已过万重山，水墨画风格"}

    # data: [plugin:]
    # data: [MSGINDEX:24]
    # data: {"type": "meta", "messageId": "676eff07-56e5-4e11-8568-181a94f0f2e5_24", "index": 24,
    #        "replyId": "676eff07-56e5-4e11-8568-181a94f0f2e5_23", "replyIndex": 23,
    #        "traceId": "be3839b9012cbd49e7a67434093adfdf", "guideId": 0, "unSupportRepeat": false}
    # data: [TRACEID:be3839b9012cbd49e7a67434093adfdf]
    # data: [DONE]

    content: str = ""

    chunk: str = ""

    crop_image: bool = False  # 是否剪裁图片

    def __init__(self, **data):
        super().__init__(**data)
        chunk = self.chunk.lstrip("data:")

        if '"type":"progress"' in chunk:
            content = json.loads(chunk).get("msg", "")

        elif '{"type":"image"' in chunk:
            chunk = json.loads(chunk)
            prompt = chunk.get("prompt", "")
            url = unquote(chunk.get("imageUrlHigh", ""))
            if self.crop_image:
                file_id = shortuuid.random()
                crop_image_and_upload(url, height=998, file_id=file_id)
                # background_task(crop_image_and_upload)(url, height=998, file_id=file_id)
                url = f"https://oss.chatfire.cn/files/{file_id}.png"

            content = f"![{prompt}]({url})\n\n"

        elif '{"type":"text"' in chunk:
            content = json.loads(chunk).get("msg", "")
        #
        # elif '{"type":"card"' in self.data:
        #     pass

        # df['title'] = "[" + df['title'] + "](" + df['url'] + ")"
        # df['image'] = "![](" + df['image'] + ")"

        else:
            content = ""
            # chunk.strip() or logger.debug(chunk)  # debug

        self.content = content


if __name__ == '__main__':
    data = """data: {"type": "text", "msg": "计算"}"""
    print(SSEData(chunk=data))



#!/usr/bin/env python
# -*- coding: utf-8 -*-
# @Project      : AI.  @by PyCharm
# @File         : v1
# @Time         : 2024/5/31 15:33
# @Author       : betterme
# @WeChat       : meutils
# @Software     : PyCharm
# @Description  :


from meutils.pipe import *

from meutils.serving.fastapi.dependencies.auth import get_bearer_token, HTTPAuthorizationCredentials
from meutils.schemas.task_types import Task

from redis.asyncio import Redis

from fastapi import APIRouter, File, UploadFile, Query, Form, BackgroundTasks, Depends, HTTPException, Request, status
from fastapi.responses import Response, FileResponse, JSONResponse

router = APIRouter()


@router.get("/tasks/{task_id}")
async def get_task(
    task_id: str,  # task:suno_id
    auth: Optional[HTTPAuthorizationCredentials] = Depends(get_bearer_token),

):
    api_key = auth and auth.credentials or None
    if api_key is None:
        raise HTTPException(status_code=status.HTTP_401_UNAUTHORIZED, detail="认证失败")

    async with Redis(decode_responses=True) as redis_client:
        data = await redis_client.get(task_id)

    return Task(task_id=task_id, data=data)


if __name__ == '__main__':
    from meutils.serving.fastapi import App

    VERSION_PREFIX = '/v1'

    app = App()
    app.include_router(router, VERSION_PREFIX)
    app.run(port=9000)



#!/usr/bin/env python
# -*- coding: utf-8 -*-
# @Project      : AI.  @by PyCharm
# @File         : muti_gpu
# @Time         : 2023/4/28 12:16
# @Author       : betterme
# @WeChat       : meutils
# @Software     : PyCharm
# @Description  : 多卡 训练 多卡加载

import os
from typing import Dict, Tuple, Union, Optional

from torch.nn import Module
from transformers import AutoModel


# os.environ["CUDA_VISIBLE_DEVICES"] = "0,1,2"

def auto_configure_device_map(num_gpus: int) -> Dict[str, int]:
    # transformer.word_embeddings 占用1层
    # transformer.final_layernorm 和 lm_head 占用1层
    # transformer.layers 占用 28 层
    # 总共30层分配到num_gpus张卡上
    num_trans_layers = 28
    per_gpu_layers = 30 / num_gpus

    # bugfix: 在linux中调用torch.embedding传入的weight,input不在同一device上,导致RuntimeError
    # windows下 model.device 会被设置成 transformer.word_embeddings.device
    # linux下 model.device 会被设置成 lm_head.device
    # 在调用chat或者stream_chat时,input_ids会被放到model.device上
    # 如果transformer.word_embeddings.device和model.device不同,则会导致RuntimeError
    # 因此这里将transformer.word_embeddings,transformer.final_layernorm,lm_head都放到第一张卡上
    device_map = {'transformer.word_embeddings': 0,
                  'transformer.final_layernorm': 0, 'lm_head': 0}

    used = 2
    gpu_target = 0
    for i in range(num_trans_layers):
        if used >= per_gpu_layers:
            gpu_target += 1
            used = 0
        assert gpu_target < num_gpus
        device_map[f'transformer.layers.{i}'] = gpu_target
        used += 1

    return device_map


def load_chatglm_on_gpus(checkpoint_path: Union[str, os.PathLike], num_gpus: int = 2,
                         device_map: Optional[Dict[str, int]] = None, **kwargs) -> Module:
    """https://github.com/THUDM/ChatGLM-6B#%E5%A4%9A%E5%8D%A1%E9%83%A8%E7%BD%B2"""
    if num_gpus < 2 and device_map is None:
        model = AutoModel.from_pretrained(checkpoint_path, trust_remote_code=True, **kwargs).half().cuda()
    else:
        from accelerate import dispatch_model

        model = AutoModel.from_pretrained(checkpoint_path, trust_remote_code=True, **kwargs).half()

        if device_map is None:
            device_map = auto_configure_device_map(num_gpus)

        model = dispatch_model(model, device_map=device_map)

    return model



#!/usr/bin/env python
# -*- coding: utf-8 -*-
# @Project      : AI.  @by PyCharm
# @File         : suno_api
# @Time         : 2024/5/29 15:43
# @Author       : betterme
# @WeChat       : meutils
# @Software     : PyCharm
# @Description  :


from meutils.pipe import *
from meutils.llm.openai_utils import appu
from meutils.db.redis_db import redis_client
from meutils.notice.feishu import send_message
from meutils.serving.fastapi.dependencies.auth import get_bearer_token, HTTPAuthorizationCredentials

from chatllm.schemas.suno_types import SunoRequest

from fastapi import APIRouter, File, UploadFile, Query, Form, Header, Depends, Request, HTTPException, status
from fastapi.responses import JSONResponse

router = APIRouter()


# @router.post("/music")
# async def e2e(
#     request: Request,
# ):
#     _ = await request.body()
#     send_message(_.decode())
#     return _
#     # return {"code": 200, "message": "success"}


@router.post("/music")
async def create_music(
    request: Request,
    auth: Optional[HTTPAuthorizationCredentials] = Depends(get_bearer_token),
    # x_api_key: Annotated[Union[List[str], None], Header()] = None,
):
    import json_repair

    request_data = json_repair.loads((await request.body()).decode())

    send_message(bjson(request_data), title="create_music")

    request = SunoRequest(**request_data)

    # logger.debug(request)
    logger.debug(request.input)

    api_key = auth and auth.credentials or None
    if api_key is None:
        detail = {
            "error": {
                "message": "",
                "type": "invalid_request_error",
                "param": None,
                "code": "invalid_api_key",
            }
        }
        raise HTTPException(status_code=status.HTTP_401_UNAUTHORIZED, detail=detail)

    async with httpx.AsyncClient(base_url="http://154.3.0.117:39955", follow_redirects=True) as client:
        if request.custom_mode:
            response = await client.post("/generate", json=request.input)

        else:
            response = await client.post("/generate/description-mode", json=request.input)

    data = response.json()
    # logger.debug(data)

    task_id = data.get("id")
    song_ids = ",".join([_.get('id') for _ in data.get("clips", [])])
    redis_client.set(f"task:{task_id}", song_ids, ex=3600 * 24 * 100)

    content = {
        "code": 200,
        "data": {
            "task_id": task_id,
        },
        "message": "success"
    }

    send_message(bjson(content), title="create_music")

    if response.is_success: await appu(api_key=api_key)
    return JSONResponse(content=content)


@router.get("/music/{task_id}")
async def get_music(
    task_id: str,
    auth: Optional[HTTPAuthorizationCredentials] = Depends(get_bearer_token),
):
    api_key = auth and auth.credentials or None
    if api_key is None:
        detail = {
            "error": {
                "message": "",
                "type": "invalid_request_error",
                "param": None,
                "code": "invalid_api_key",
            }
        }
        raise HTTPException(status_code=status.HTTP_401_UNAUTHORIZED, detail=detail)

    ids = redis_client.get(f"task:{task_id}").decode().split(',')
    clips = {}
    status_set = set()
    async with httpx.AsyncClient(base_url="http://154.3.0.117:39955", follow_redirects=True) as client:
        for song_id in ids:
            response = await client.get(f"/feed/{song_id}")
            clip = response.json()[0]
            clips[song_id] = clip
            status_set.add(clip.get("status"))

    content = {
        "code": 200,
        "data": {
            "task_id": task_id,
            "status": ",".join(status_set) + "d",
            "input": str(clip.get("metadata")),
            "clips": clips,
        },
        "message": "success"
    }
    if response.is_success and "complete" in status_set: await appu('ppu-0001', api_key=api_key)
    return JSONResponse(content=content)


if __name__ == '__main__':
    from meutils.serving.fastapi import App

    app = App()

    app.include_router(router, '/v1/suno/v1')

    app.run()



#!/usr/bin/env python
# -*- coding: utf-8 -*-
# @Project      : AI.  @by PyCharm
# @File         : v1
# @Time         : 2024/5/31 13:29
# @Author       : betterme
# @WeChat       : meutils
# @Software     : PyCharm
# @Description  :


from meutils.pipe import *
from meutils.llm.openai_utils import appu
from meutils.serving.fastapi.dependencies.auth import get_bearer_token, HTTPAuthorizationCredentials
from chatllm.schemas.openai_types import BatchRequest

from openai import OpenAI, AsyncOpenAI

from fastapi import APIRouter, File, UploadFile, Query, Form, BackgroundTasks, Depends, HTTPException, Request, status
from fastapi.responses import Response, FileResponse, JSONResponse, RedirectResponse

router = APIRouter()

zhipu_client = AsyncOpenAI(
    api_key=os.getenv('ZHIPU_API_KEY'),
    base_url=os.getenv('ZHIPU_BASE_URL'),
)


@router.post("/batches")
async def create_batch(
    request: BatchRequest,
    auth: Optional[HTTPAuthorizationCredentials] = Depends(get_bearer_token),
):
    logger.debug(request)

    api_key = auth and auth.credentials or None
    if api_key is None:
        raise HTTPException(status_code=status.HTTP_401_UNAUTHORIZED, detail="认证失败")

    request.endpoint = request.endpoint.replace("/v1/chat/completions", "/v4/chat/completions")
    batch_object = await zhipu_client.batches.create(**request.model_dump(), )

    # await appu("ppu-01", api_key)
    return batch_object


@router.get("/batches/{batch_id}")
async def retrieve_batch(
    batch_id: str,
    auth: Optional[HTTPAuthorizationCredentials] = Depends(get_bearer_token)
):
    api_key = auth and auth.credentials or None
    if api_key is None:
        raise HTTPException(status_code=status.HTTP_401_UNAUTHORIZED, detail="认证失败")
    batch_object = await zhipu_client.batches.retrieve(batch_id)

    # await appu("ppu-001", api_key)
    return batch_object


# @router.get(
#     "/batches",
#     responses={
#         200: {"model": ListBatchesResponse, "description": "Batch listed successfully."},
#     },
#     tags=["Batch"],
#     summary="List your organization&#39;s batches.",
#     response_model_by_alias=True,
# )
# async def list_batches(
#     after: str = Query(None,
#                        description="A cursor for use in pagination. &#x60;after&#x60; is an object ID that defines your place in the list. For instance, if you make a list request and receive 100 objects, ending with obj_foo, your subsequent call can include after&#x3D;obj_foo in order to fetch the next page of the list. ")
#     ,
#     limit: int = Query(20,
#                        description="A limit on the number of objects to be returned. Limit can range between 1 and 100, and the default is 20. ")
#     ,
#     token_ApiKeyAuth: TokenModel = Security(
#         get_token_ApiKeyAuth
#     ),
# ) -> ListBatchesResponse:
#     ...
#
# @router.post(
#     "/batches/{batch_id}/cancel",
#     responses={
#         200: {"model": Batch, "description": "Batch is cancelling. Returns the cancelling batch&#39;s details."},
#     },
#     tags=["Batch"],
#     summary="Cancels an in-progress batch.",
#     response_model_by_alias=True,
# )
# async def cancel_batch(
#     batch_id: str = Path(..., description="The ID of the batch to cancel.")
#     ,
#     token_ApiKeyAuth: TokenModel = Security(
#         get_token_ApiKeyAuth
#     ),
# ) -> Batch:
#     ...

if __name__ == '__main__':
    from meutils.serving.fastapi import App

    VERSION_PREFIX = '/v1'

    app = App()
    app.include_router(router, VERSION_PREFIX)
    app.run(port=8000)



#!/usr/bin/env python
# -*- coding: utf-8 -*-
# @Project      : AI.  @by PyCharm
# @File         : keys4openai
# @Time         : 2024/5/14 16:42
# @Author       : betterme
# @WeChat       : meutils
# @Software     : PyCharm
# @Description  : https://github.com/CZT0/StableOpenAI/tree/main
"""
1. 如果有需要增加tokens计算，可放大
2. 平滑+前后置内容
3. 淘汰策略+轮询策略
    前置淘汰
    后置淘汰：根据具体错误
4. 统计信息
5. 日志监控
"""
import functools
import os

from meutils.pipe import *
from meutils.config_utils.lark_utils import get_spreadsheet_values
from meutils.notice.feishu import send_message
from meutils.db.redis_db import redis_aclient

from chatllm.schemas.openai_api_protocol import ChatCompletionRequest
from openai import AsyncOpenAI, OpenAI

import redis


# np.random.choice()

class PollKeys(object):  # 轮询

    def __init__(self, provider='kimi', rpm: int = 3):
        self.provider = provider
        self.base_url = f"https://any2chat.chatfire.cn/{self.provider}"
        self.delay = 60 // rpm
        self.redis_key = f"api-provider:{provider}"

    async def acreate(self, request: ChatCompletionRequest):
        for i in range(3):  # 轮询个数
            api_key = self.api_key

            try:
                completion = AsyncOpenAI(
                    base_url=self.base_url,
                    api_key=api_key,
                ).chat.completions.create(**request.model_dump())

                return await completion

            except Exception as e:
                await redis_aclient.hincrby(self.redis_key, api_key, 1)  # 错误计数
                # todo：次数超过阈值，淘汰
                api_key_counter = await redis_aclient.hgetall(self.redis_key)

                logger.error(e)
                send_message(f"{api_key_counter.decode()}: {e}", title=self.provider)

    @property
    async def api_key(self):
        """获取一个api_key"""
        while 1:
            api_key = await redis_aclient.lpop(self.provider)
            await redis_aclient.rpush(self.provider, api_key)

            return api_key

    @diskcache(location='check_api_key_cache', ttl=15 * 60, ignore=['self'])
    async def check_api_key(self, api_key):
        """后台定时检测：半小时检测一遍
        1、被禁用
        2、无余额

        """
        logger.debug("@@@@@@@@2")
        payload = {
            "token": api_key  # "eyJhbGciOiJIUzUxMiIsInR5cCI6IkpXVCJ9..."
        }
        response = httpx.Client(base_url=self.base_url).post("/token/check", json=payload)
        _ = response.json()

        # 淘汰key发出警告
        message = f"{api_key}: {_}"
        logger.warning(message)
        send_message(message, title=f"{self.provider}_check_api_key")

        return _.get("live", False)

    def get_and_update_api_keys(self):  # 保活
        """定时更新&空闲更新
                1、 通过飞书更新keys
                2、检测key
                3、更新key


                调用 => key有效性检查 => 当前key `60 // rpm +1` s后，放入keys右侧 => 轮询
                """
        # 获取
        logger.debug(feishu_url := os.getenv(f"{self.provider.upper()}_KEYS_URL", "DEEPSEEK_KEYS_URL"))

        df = get_spreadsheet_values(
            feishu_url=feishu_url,
            to_dataframe=True)
        api_keys = filter(None, df[0])
        api_keys = filter(self.check_api_key, api_keys)

        # 更新
        self.redis_client.delete(self.provider)
        self.redis_client.rpush(self.provider, *api_keys)

        return api_keys


if __name__ == '__main__':
    print(str(PollKeys().check_api_key('')))



#!/usr/bin/env python
# -*- coding: utf-8 -*-
# @Project      : AI.  @by PyCharm
# @File         : qa
# @Time         : 2023/8/11 08:50
# @Author       : betterme
# @WeChat       : meutils
# @Software     : PyCharm
# @Description  :

from meutils.pipe import *
from langchain_community.embeddings.openai import OpenAIEmbeddings
from langchain.text_splitter import RecursiveCharacterTextSplitter
from langchain_community.vectorstores import Chroma
from langchain.chains import RetrievalQAWithSourcesChain
from langchain_community.chat_models import ChatOpenAI
from langchain.prompts.chat import (
    ChatPromptTemplate,
    SystemMessagePromptTemplate,
    HumanMessagePromptTemplate,
)
import os
import chainlit as cl

os.environ["OPENAI_API_KEY"] = "OPENAI_API_KEY"

text_splitter = RecursiveCharacterTextSplitter(chunk_size=1000, chunk_overlap=100)

system_template = """Use the following pieces of context to answer the users question.
If you don't know the answer, just say that you don't know, don't try to make up an answer.
ALWAYS return a "SOURCES" part in your answer.
The "SOURCES" part should be a reference to the source of the document from which you got your answer.

Example of your response should be:

```
The answer is foo
SOURCES: xyz
```

Begin!
----------------
{summaries}"""
messages = [
    SystemMessagePromptTemplate.from_template(system_template),
    HumanMessagePromptTemplate.from_template("{question}"),
]
prompt = ChatPromptTemplate.from_messages(messages)
chain_type_kwargs = {"prompt": prompt}


@cl.on_chat_start
async def init():
    files = None

    # Wait for the user to upload a file
    while files == None:
        files = await cl.AskFileMessage(
            content="Please upload a text file to begin!", accept=["text/plain"]
        ).send()

    file = files[0]

    msg = cl.Message(content=f"Processing `{file.name}`...")
    await msg.send()

    # Decode the file
    text = file.content.decode("utf-8")

    # Split the text into chunks
    texts = text_splitter.split_text(text)

    # Create a metadata for each chunk
    metadatas = [{"source": f"{i}-pl"} for i in range(len(texts))]

    # Create a Chroma vector store
    embeddings = OpenAIEmbeddings()
    docsearch = await cl.make_async(Chroma.from_texts)(
        texts, embeddings, metadatas=metadatas
    )
    # Create a chain that uses the Chroma vector store
    chain = RetrievalQAWithSourcesChain.from_chain_type(
        ChatOpenAI(temperature=0, streaming=True),
        chain_type="stuff",
        retriever=docsearch.as_retriever(),
    )

    # Save the metadata and texts in the user session
    cl.user_session.set("metadatas", metadatas)
    cl.user_session.set("texts", texts)

    # Let the user know that the system is ready
    msg.content = f"Processing `{file.name}` done. You can now ask questions!"
    await msg.update()

    cl.user_session.set("chain", chain)


@cl.on_message
async def main(message):
    chain = cl.user_session.get("chain")  # type: RetrievalQAWithSourcesChain
    cb = cl.AsyncLangchainCallbackHandler(
        stream_final_answer=True, answer_prefix_tokens=["FINAL", "ANSWER"]
    )
    cb.answer_reached = True
    res = await chain.acall(message, callbacks=[cb])

    answer = res["answer"]
    sources = res["sources"].strip()
    source_elements = []

    # Get the metadata and texts from the user session
    metadatas = cl.user_session.get("metadatas")
    all_sources = [m["source"] for m in metadatas]
    texts = cl.user_session.get("texts")

    if sources:
        found_sources = []

        # Add the sources to the message
        for source in sources.split(","):
            source_name = source.strip().replace(".", "")
            # Get the index of the source
            try:
                index = all_sources.index(source_name)
            except ValueError:
                continue
            text = texts[index]
            found_sources.append(source_name)
            # Create the text element referenced in the message
            source_elements.append(cl.Text(content=text, name=source_name))

        if found_sources:
            answer += f"\nSources: {', '.join(found_sources)}"
        else:
            answer += "\nNo sources found"

    if cb.has_streamed_final_answer:
        cb.final_stream.elements = source_elements
        await cb.final_stream.update()
    else:
        await cl.Message(content=answer, elements=source_elements).send()



#!/usr/bin/env python
# -*- coding: utf-8 -*-
# @Project      : AI.  @by PyCharm
# @File         : files
# @Time         : 2023/12/29 14:21
# @Author       : betterme
# @WeChat       : meutils
# @Software     : PyCharm
# @Description  : todo: 返回链接，参考kimi，接入文档解析，16c32g机器部署，不能通服务调用, minio_client通用化 id 映射

from meutils.pipe import *

from meutils.oss.minio_oss import Minio  # minio_client
from meutils.db.redis_db import redis_aclient  # minio_client
from meutils.llm.openai_utils import appu

from meutils.serving.fastapi.dependencies.auth import get_bearer_token, HTTPAuthorizationCredentials

# from chatllm.llmchain.document_loaders.file_loader import UnstructuredAPIFileLoader
from chatllm.llmchain.completions import chatglm_web

from enum import Enum

from openai import OpenAI, AsyncOpenAI
from openai._types import FileTypes
from openai.types.file_object import FileObject
from fastapi import APIRouter, File, UploadFile, Query, Form, BackgroundTasks, Depends, HTTPException, Request, status
from fastapi.responses import Response, FileResponse, JSONResponse

router = APIRouter()

kimi_client = AsyncOpenAI(
    api_key=os.getenv('MOONSHOT_API_KEY'),
    base_url=os.getenv('MOONSHOT_BASE_URL'),
)

zhipu_client = AsyncOpenAI(
    api_key=os.getenv('ZHIPU_API_KEY'),
    base_url=os.getenv('ZHIPU_BASE_URL'),
)


class Purpose(str, Enum):
    file_upload = "file-upload"
    file_upload_minio = "file-upload-minio"

    # file_upload_glm = "file-upload-glm"
    file_upload_chatpdf = "file-upload-chatpdf"

    file_extract = "file-extract"  # kimi glm
    # file_extract_kimi = "file-extract-kimi"
    #
    # file_extract_plus = "file-extract-plus"  # 自研
    #
    # rag = "rag"
    # file_structuring = "file_structuring"
    # file_embedding = "file_embedding"
    #
    # assistants = "assistants"
    # fine_tune = "fine-tune"
    batch = "batch"


OPENAI_BUCKET = os.getenv('OPENAI_BUCKET', 'files')


@router.post("/files")  # 同名文件会被覆盖
async def upload_files(
    file: Union[UploadFile] = File(...),  # 文件链接
    purpose: Purpose = Form(...),
    auth: Optional[HTTPAuthorizationCredentials] = Depends(get_bearer_token),
):
    api_key = auth and auth.credentials or None
    if api_key is None:
        raise HTTPException(status_code=status.HTTP_401_UNAUTHORIZED, detail="认证失败")

    if purpose == Purpose.file_upload:  # 单纯的上传
        try:
            file_object = await chatglm_web.Completions().put_object_for_openai(file=file, purpose="file-upload")

            await appu("ppu-001", api_key)
            return file_object
        except Exception as e:
            logger.error(e)

            file_object = await Minio().put_object_for_openai(file=file, purpose=purpose.value,
                                                              bucket_name=OPENAI_BUCKET)
            await appu("ppu-001", api_key)
            return file_object

    elif purpose == Purpose.file_upload_minio:
        file_object = await Minio().put_object_for_openai(file=file, purpose=purpose.value,
                                                          bucket_name=OPENAI_BUCKET)
        await appu("ppu-001", api_key)
        return file_object


    elif purpose == Purpose.batch:
        if not file.filename.endswith('jsonl'):
            raise HTTPException(
                status_code=status.HTTP_400_BAD_REQUEST,
                detail={'error': {'code': '1622', 'message': f'文件内容异常:文件名{file.filename}扩展名只能为jsonl'}}
            )

        file_object = await zhipu_client.files.create(file=(file.filename, file.file), purpose="batch")
        await appu("ppu-01", api_key)
        return file_object

    # elif purpose == Purpose.file_extract_plus:
    #     content = await UnstructuredAPIFileLoader.load_for_openai(file)

    elif purpose == Purpose.file_extract:
        file_object = await kimi_client.files.create(file=(file.filename, file.file), purpose=purpose.value)

        await appu("ppu-01", api_key)
        return file_object

    # elif purpose == Purpose.file_extract_kimi:
    #     await appu("ppu-01", api_key)
    #     return await kimi_client.files.create(file=(file.filename, file.file), purpose=purpose.value)

    elif purpose == Purpose.file_upload_chatpdf:
        file_object = await chatglm_web.Completions().put_object_for_openai(file=file, purpose="chatpdf-file-upload")

        await appu("ppu-01", api_key)
        return file_object

    # elif purpose == Purpose.rag:
    #     raise HTTPException(status_code=status.HTTP_501_NOT_IMPLEMENTED, detail="暂不支持")


@router.get("/files/{file_id}/content")
async def get_files_content(
    file_id: str,
    auth: Optional[HTTPAuthorizationCredentials] = Depends(get_bearer_token),

):
    api_key = auth and auth.credentials or None
    if api_key is None:
        raise HTTPException(status_code=status.HTTP_401_UNAUTHORIZED, detail="认证失败")

    # 缓存
    content = await redis_aclient.get(file_id)

    # kimi
    if len(file_id) == 20:  # cpeni8onsmmqhvjjv5p0 好像是c开头，先用长度判断
        if content is None:
            response = await kimi_client.files.content(file_id=file_id)
            content = response.text
            # 缓存
            await redis_aclient.set(file_id, content, ex=3600 * 24 * 180)  # 180天
            await kimi_client.files.delete(file_id)  # 删除文件

    # if content is None:
    #     url = f"https://f.chatllm.vip/{OPENAI_BUCKET}/{file_id}"
    #     content = await UnstructuredAPIFileLoader.load_for_openai(url)
    #     content = json.dumps(content, ensure_ascii=False)
    #
    #     # response = await Minio().aget_object(OPENAI_BUCKET, file_id)
    #     # response = Minio().get_object(OPENAI_BUCKET, file_id)
    #     # content = response.read()
    #     # file = io.BytesIO(content)

    # 解析内容不支持 files.content(file_id).stream_to_file('xx.pdf')
    return Response(content=content, media_type="application/json")  # json.dumps(content, ensure_ascii=False)


@router.get("/files/{file_id}")
async def get_files(
    file_id: str,
    auth: Optional[HTTPAuthorizationCredentials] = Depends(get_bearer_token),
):
    api_key = auth and auth.credentials or None
    if api_key is None:
        raise HTTPException(status_code=status.HTTP_401_UNAUTHORIZED, detail="认证失败")

    if file_id.startswith("cn"):  # kimi
        return await kimi_client.files.retrieve(file_id=file_id)

    # 映射 file_id
    # response = Minio().get_object(OPENAI_BUCKET, file_id)
    response = await Minio().aget_object(OPENAI_BUCKET, file_id)
    data = response.read()

    return FileObject(
        id=file_id,
        bytes=len(data),
        created_at=int(time.time()),
        filename=file_id,  # filename
        object='file',
        purpose='assistants',
        status='processed',
    )


@router.delete("/files/{file_id}")
async def delete_file(
    file_id: str,
    auth: Optional[HTTPAuthorizationCredentials] = Depends(get_bearer_token),
):
    api_key = auth and auth.credentials or None
    if api_key is None:
        raise HTTPException(status_code=status.HTTP_401_UNAUTHORIZED, detail="认证失败")

    if file_id.startswith("cn"):  # kimi
        response = await kimi_client.files.delete(file_id=file_id)
        return response


if __name__ == '__main__':
    from meutils.serving.fastapi import App

    VERSION_PREFIX = '/v1'

    app = App()
    app.include_router(router, VERSION_PREFIX)
    app.run(port=9000)



#!/usr/bin/env python
# -*- coding: utf-8 -*-
# @Project      : AI.  @by PyCharm
# @File         : suno
# @Time         : 2024/3/28 12:41
# @Author       : betterme
# @WeChat       : meutils
# @Software     : PyCharm
# @Description  :
import httpx

from meutils.pipe import *
from meutils.cache_utils import ttl_cache
from meutils.decorators.retry import retrying
from meutils.queues.smooth_queue import SmoothQueue

from meutils.notice.feishu import send_message

from openai.types.chat.chat_completion import ChatCompletion
from openai.types.chat.chat_completion_chunk import ChatCompletionChunk

from chatllm.llmchain.completions import openai_completions
from chatllm.schemas.openai_api_protocol import ChatCompletionRequest, UsageInfo
from chatllm.schemas.suno_types import SunoRequest

from chatllm.utils.openai_utils import to_openai_completion_params, openai_response2sse


# url = f"https://studio-api.suno.ai/api/feed/?ids={id1}%2C{id2}"
# 异步信息

class Completions(object):
    def __init__(self, **client_params):
        self.api_key = client_params.get('api_key')
        self.access_token = self.get_access_token(api_key=self.api_key)
        # self.client = httpx.Client(headers=self.headers, base_url="https://studio-api.suno.ai", follow_redirects=True)
        self.base_url = "https://studio-api.suno.ai"
        self.client = httpx.Client(base_url=self.base_url, follow_redirects=True)

    async def acreate(self, request: ChatCompletionRequest, **kwargs):
        request.model = request.model.startswith('gpt-4') and 'gpt-4' or 'gpt-3.5-turbo'
        data = to_openai_completion_params(request)

        start_time = time.perf_counter()

        try:
            _ = await self.client.chat.completions.create(**data)
        except Exception as e:
            logger.error(e)
            send_message(f"{self.api_key}\n{e}\n耗时：{time.perf_counter() - start_time} s",
                         "Copilot_token异常或者请求异常")

            # 备用
            data['model'] = 'backup-gpt'
            _ = await AsyncOpenAI().chat.completions.create(**data)

        # todo: 平滑操作
        if request.stream and data['model'].startswith('gpt-4'):
            return SmoothQueue().consumer(_, delay=0.055)

        return _

    def create(self, request: ChatCompletionRequest, **kwargs):
        prompts = request.messages[-1].get("content", "随便写一首歌").strip().split(maxsplit=1)

        suno = SunoRequest(title=prompts[0], song_description=prompts[-1])

        logger.debug(suno)

        # "https://cdn1.suno.ai/{song_id}.mp3"
        # "https://cdn1.suno.ai/{song_id}.mp3"

        # httpx.post("https://studio-api.suno.ai/api/generate/v2", data=suno.dict(), headers=c.get_headers(), follow_redirects=True)

        response = self.client.post("/api/generate/v2/", json=suno.dict(), headers=self.get_headers())

        # self.client.get
        return response.json()

    @property
    def total_credits_left(self) -> int:
        total_credits_left = self.client.get('/api/billing/info', headers=self.get_headers()).json().get(
            "total_credits_left")
        return total_credits_left

    @staticmethod
    @retrying(max_retries=1)
    @ttl_cache(ttl=60)
    def get_access_token(api_key: Optional[str] = None, dynamic_param=None):
        api_key = api_key or "eyJhbGciOiJSUzI1NiIsInR5cCI6IkpXVCJ9.eyJpZCI6ImNsaWVudF8yZUluMHBQZEFFWlpoNjVtY0hvNFpncG95SkoiLCJyb3RhdGluZ190b2tlbiI6IjR5Z2R4dHR2ODY4OWh6dm56cWM4cWhkdnVpeDY1a21mMzZ0ajJtc2MifQ.lmzIvyYCCMQ_vNdGJO_WKPG2ptjK3gj16is5PUy0TRdQKjTj05BJCfXg5IMCWURghIFC5C4mKdzfKrz7ld0AMclvecc87NeKVtPB4WIHaoV9PN_eIkCT6heJHgd6fp_dhrlqNAI6NJtIewbfko4Q9RcYsJYyfesiKY5FhzEvphfzX3Og35luV8COSNfRfbu3Kv6zpifNIDAhGPHtjT4vQo6TucCsO4vjbNl0PYgJHYMDtmUxaq8bppxj2lcORen94w5oy33zqHE1O0UGFnE248sYib-LLeuoTWnH0uvDe03ovGRJ1H0Tk1e8fiKC4Y0q90zAe46FiyEV0I9V_yUdUA"
        headers = {
            "User-Agent": "Mozilla/5.0 (Windows NT 10.0; Win64; x64) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/117.0.0.0 Safari/537.36",
            "Cookie": f"__client={api_key}"
        }

        with httpx.Client() as client:
            url = "https://clerk.suno.ai/v1/client?_clerk_js_version=4.70.5"
            response = client.get(url, headers=headers).json()
            sid = response.get("response", {}).get("last_active_session_id")

            url = f"https://clerk.suno.ai/v1/client/sessions/{sid}/tokens?_clerk_js_version=4.70.5"
            response = client.post(url, headers=headers)  # refresh

        resp = response.json()
        resp['api_key'] = api_key
        _ = resp.get('jwt')  # token

        # logger.debug(resp)

        if not _:
            send_message(str(resp), "suno_token 异常")

        return _

    def get_headers(self):
        access_token = self.get_access_token(api_key=self.api_key)

        COMMON_HEADERS = {
            'Content-Type': 'text/plain;charset=UTF-8',
            'User-Agent': 'Mozilla/5.0 (Macintosh; Intel Mac OS X 10_15_7) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/123.0.0.0 Safari/537.36',
            "Referer": "https://app.suno.ai/",
            "Origin": "https://app.suno.ai",
        }

        headers = {
            "Accept-Encoding": "gzip, deflate, br",
            "User-Agent": "Mozilla/5.0 (Windows NT 10.0; Win64; x64) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/117.0.0.0 Safari/537.36",
            "Authorization": f"Bearer {access_token}"
        }
        return {**COMMON_HEADERS, **headers}


if __name__ == '__main__':
    with timer():
        c = Completions()
        print(c.total_credits_left)

    prompt = """
        如何鉴别渣女
        瑜伽裤 健身房
        酒店 拍照 躺在床
        头等舱 上三亚
        露光背 拍一下
        秀厨艺 下午茶
        小小绿茶 你别惊讶
        朋友圈高逼格 银行卡里没余额
        约个会 黑丝腿
        """
    request = ChatCompletionRequest(messages=[{'role': 'user', 'content': prompt}])

    r = c.create(request)
    # df = pd.DataFrame(s['clips'])
    # df[['video_url', 'audio_url', 'image_url', 'model_name', 'created_at', 'metadata']]
    # metadata: audio_url、video_url、duration
    # s  = {'id': 'fbd295bb-1efd-453b-9ed4-88e9eadf6940', 'clips': [
    #     {'id': 'd8d813df-db75-44d5-9640-5330ee1e25af', 'video_url': '', 'audio_url': '', 'image_url': None,
    #      'image_large_url': None, 'major_model_version': 'v3', 'model_name': 'chirp-v3',
    #      'metadata': {'tags': 'pop', 'prompt': '', 'gpt_description_prompt': None, 'audio_prompt_id': None,
    #                   'history': None, 'concat_history': None, 'type': 'gen', 'duration': None, 'refund_credits': None,
    #                   'stream': True, 'error_type': None, 'error_message': None}, 'is_liked': False,
    #      'user_id': '6275effb-094c-439a-8218-914fb33444ff', 'is_trashed': False, 'reaction': None,
    #      'created_at': '2024-03-28T12:50:09.193Z', 'status': 'submitted', 'title': '如何鉴别渣女', 'play_count': 0,
    #      'upvote_count': 0, 'is_public': False},
    #     {'id': '77de0e97-2fe6-40ca-8494-210cad1168d8', 'video_url': '', 'audio_url': '', 'image_url': None,
    #      'image_large_url': None, 'major_model_version': 'v3', 'model_name': 'chirp-v3',
    #      'metadata': {'tags': 'pop', 'prompt': '', 'gpt_description_prompt': None, 'audio_prompt_id': None,
    #                   'history': None, 'concat_history': None, 'type': 'gen', 'duration': None, 'refund_credits': None,
    #                   'stream': True, 'error_type': None, 'error_message': None}, 'is_liked': False,
    #      'user_id': '6275effb-094c-439a-8218-914fb33444ff', 'is_trashed': False, 'reaction': None,
    #      'created_at': '2024-03-28T12:50:09.193Z', 'status': 'submitted', 'title': '如何鉴别渣女', 'play_count': 0,
    #      'upvote_count': 0, 'is_public': False}],
    #  'metadata': {'tags': 'pop', 'prompt': '', 'gpt_description_prompt': None, 'audio_prompt_id': None, 'history': None,
    #               'concat_history': None, 'type': 'gen', 'duration': None, 'refund_credits': None, 'stream': True,
    #               'error_type': None, 'error_message': None}, 'major_model_version': 'v3', 'status': 'running',
    #  'created_at': '2024-03-28T12:50:09.170Z', 'batch_size': 2}



#!/usr/bin/env python
# -*- coding: utf-8 -*-
# @Project      : AI.  @by PyCharm
# @File         : ark
# @Time         : 2024/5/15 15:26
# @Author       : betterme
# @WeChat       : meutils
# @Software     : PyCharm
# @Description  : https://www.volcengine.com/product/doubao?utm_source=5&utm_medium=sembaidu&utm_term=sem_baidu_damoxing_doubao_1&utm_campaign=vgbdpcztn797661721486&utm_content=damoxing_doubao
import os

from meutils.pipe import *
import volcenginesdkcore
import volcenginesdkark
from pprint import pprint
from volcenginesdkcore.rest import ApiException

endpoint_map ={
            "doubao-lite-128k": "ep-20240520010805-566f7",
            "doubao-lite-32k": "ep-20240515062431-rkhfp",
            "doubao-lite-4k": "ep-20240515062545-nnq8b",

            "doubao-pro-128k": "ep-20240515073409-dlpqp",
            "doubao-pro-32k": "ep-20240515073434-s4skk",
            "doubao-pro-4k": "ep-20240515073524-xj4m2",  # fc

            "moonshot-v1-8k": "ep-20240516010405-mwthf",
            "moonshot-v1-32k": "ep-20240516010345-f88rs",
            "moonshot-v1-128k": "ep-20240516010323-zfzwj",

            "doubao-embedding": "ep-20240516005609-9c9pq",
        }

if __name__ == '__main__':
    configuration = volcenginesdkcore.Configuration()
    configuration.ak = os.getenv("ARK_ACCESS_KEY")
    configuration.sk = os.getenv("ARK_SECRET_ACCESS_KEY")
    configuration.region = "cn-beijing"
    # set default configuration
    volcenginesdkcore.Configuration.set_default(configuration)

    # use global default configuration
    api_instance = volcenginesdkark.ARKApi()

    # for endpoint in endpoint_map.values():
    get_api_key_request = volcenginesdkark.GetApiKeyRequest(
        duration_seconds=30 * 24 * 3600,
        resource_type="endpoint",
        resource_ids=list(endpoint_map.values()),
        # resource_ids=["ep-20240515062545-nnq8b"]
    )




    try:
        resp = api_instance.get_api_key(get_api_key_request)
        pprint(resp)
    except ApiException as e:
        print("Exception when calling api: %s\n" % e)



    '''
    Usage:

    1. python3 -m pip install --user volcengine
    2. python main.py
    '''
    import os
    from volcengine.maas.v2 import MaasService
    from volcengine.maas import MaasException, ChatRole


    def test_chat(maas, endpoint_id, req):
        try:
            resp = maas.chat(endpoint_id, req)
            print(resp)
        except MaasException as e:
            print(e)


    def test_stream_chat(maas, endpoint_id, req):
        try:
            resps = maas.stream_chat(endpoint_id, req)
            for resp in resps:
                print(resp)
        except MaasException as e:
            print(e)


        maas = MaasService('maas-api.ml-platform-cn-beijing.volces.com', 'cn-beijing')

        # set ak&sk
        maas.set_ak("VOLC_ACCESSKEY")
        maas.set_sk("VOLC_SECRETKEY")
        # r := client.NewInstance("maas-api.ml-platform-cn-beijing.volces.com", "cn-beijing")

        # chat
        req = {
            "parameters": {
                "max_new_tokens": 1024,  # 输出文本的最大tokens限制，max_new_tokens + input_length <= max_input_size
                "temperature": 0.3,  # 用于控制生成文本的随机性和创造性，Temperature值越大随机性越大，取值范围0~1
                "top_p": 0.9,  # 用于控制输出tokens的多样性，TopP值越大输出的tokens类型越丰富，取值范围0~1
            },
            "messages": [
                {
                    "role": ChatRole.USER,
                    "content": "天为什么这么蓝？"
                }, {
                    "role": ChatRole.ASSISTANT,
                    "content": "因为有你"
                }, {
                    "role": ChatRole.USER,
                    "content": "花儿为什么这么香？"
                },
            ]
        }

        endpoint_id = 'ep-20240516010405-mwthf'
        test_chat(maas, endpoint_id, req)
        test_stream_chat(maas, endpoint_id, req)



#!/usr/bin/env python
# -*- coding: utf-8 -*-
# @Project      : AI.  @by PyCharm
# @File         : chatbase
# @Time         : 2023/7/5 15:29
# @Author       : betterme
# @WeChat       : meutils
# @Software     : PyCharm
# @Description  :

from meutils.pipe import *
from langchain.schema import Document

from langchain_community.chat_models import ChatOpenAI
from langchain.cache import InMemoryCache
from langchain.memory import ConversationBufferWindowMemory
from langchain_community.embeddings import OpenAIEmbeddings
from langchain.embeddings.base import Embeddings
from langchain_community.vectorstores import VectorStore, DocArrayInMemorySearch, Zilliz, FAISS

from langchain.chains.question_answering import load_qa_chain
from langchain.chains.qa_with_sources import load_qa_with_sources_chain  # 输出SOURCE废token
from langchain.chains import ConversationChain
from langchain_community.document_loaders import DirectoryLoader, PyMuPDFLoader


# import langchain
#
# langchain.verbose = True
# langchain.debug = True


class ChatBase(object):
    """
    ChatBase().create_index().search().run(query='1+1')
    """

    def __init__(self, model="gpt-3.5-turbo", embeddings: Embeddings = OpenAIEmbeddings(chunk_size=100), k=1,
                 temperature=0):
        self.memory = ConversationBufferWindowMemory(memory_key="chat_history", return_messages=True, k=k)
        self.memory_messages = self.memory.chat_memory.messages
        self.embeddings = embeddings  # todo: 本地向量
        self.llm = ChatOpenAI(model=model, temperature=temperature, streaming=True)
        self.chain = load_qa_chain(self.llm, chain_type="stuff")  # map_rerank 重排序
        #
        self._docs = None
        self._index = None
        self._input = None

    def create_index(self, docs: List[Document], vectorstore: VectorStore = DocArrayInMemorySearch):  # 主要耗时，缓存是否生效
        self._index = vectorstore.from_documents(docs, self.embeddings)  # 向量阶段：可以多线程走缓存？
        return self

    def search(self, query, k: int = 5, threshold: float = 0.7, **kwargs):
        docs_scores = self._index.similarity_search_with_score(query, k=k, **kwargs)
        self._docs = []
        for doc, score in docs_scores:
            if score > threshold:
                doc.metadata['score'] = score
                doc.metadata['page_content'] = doc.page_content
                self._docs.append(doc)

        self._input = {"input_documents": self._docs, "question": query}  # todo: input_func

        return self

    def run(self):
        return self.chain.run(self._input)  # 流式




#!/usr/bin/env python
# -*- coding: utf-8 -*-
# @Project      : AI.  @by PyCharm
# @File         : completions
# @Time         : 2023/12/19 16:38
# @Author       : betterme
# @WeChat       : meutils
# @Software     : PyCharm
# @Description  : 逆向工程

from meutils.pipe import *
from meutils.notice.feishu import send_message
from meutils.serving.fastapi.dependencies.auth import get_bearer_token, HTTPAuthorizationCredentials

from sse_starlette import EventSourceResponse
from fastapi import APIRouter, File, UploadFile, Query, Form, Depends, Request, HTTPException, status

from openai import OpenAI, AsyncOpenAI
from openai.types.chat import ChatCompletion, ChatCompletionChunk

from chatllm.llmchain.completions import chatglm_web
from chatllm.schemas.openai_api_protocol import ChatCompletionRequest, UsageInfo
from chatllm.utils.openai_utils import openai_response2sse

router = APIRouter()

ChatCompletionResponse = Union[ChatCompletion, List[ChatCompletionChunk]]


@router.post("/chat/completions")
async def create_chat_completions(
    request: ChatCompletionRequest,
    auth: Optional[HTTPAuthorizationCredentials] = Depends(get_bearer_token),
):
    logger.debug(request)

    api_key = auth and auth.credentials or None
    if api_key is None:
        detail = {
            "error": {
                "message": "",
                "type": "invalid_request_error",
                "param": None,
                "code": "invalid_api_key",
            }
        }
        raise HTTPException(status_code=status.HTTP_401_UNAUTHORIZED, detail=detail)

    # 轮询
    if "|" in api_key:
        api_key = np.random.choice(api_key.strip().split('|'))

    if request.model.startswith(("test", "chatfire")):
        completions = chatglm_web.Completions()
    else:
        completions = chatglm_web.Completions(api_key=api_key)

    response = completions.create_sse(request)
    return response


if __name__ == '__main__':
    from meutils.serving.fastapi import App

    app = App()

    app.include_router(router, '/v1')

    app.run()
    # for i in range(10):
    #     send_message(f"兜底模型", title="github_copilot")



#!/usr/bin/env python
# -*- coding: utf-8 -*-
# @Project      : AI.  @by PyCharm
# @File         : ark_completions
# @Time         : 2024/5/16 09:50
# @Author       : betterme
# @WeChat       : meutils
# @Software     : PyCharm
# @Description  :

from meutils.pipe import *

from chatllm.schemas.openai_api_protocol import ChatCompletionRequest, UsageInfo
from chatllm.utils.openai_utils import openai_response2sse, to_openai_completion_params

from openai import OpenAI, AsyncOpenAI


class Completions(object):
    def __init__(self, api_key=None):
        api_key = self.get_and_update_api_key(resource_ids=list(self.endpoint_map.values()))

        params = dict(
            api_key=api_key,
            base_url=os.getenv("ARK_BASE_URL", "https://ark.cn-beijing.volces.com/api/v3"),
            max_retries=1,
        )
        # 如果这一步就报错呢
        self.client = OpenAI(**params)
        self.aclient = AsyncOpenAI(**params)

    def create(self, request: ChatCompletionRequest):
        request.model = self.endpoint_map.get(request.model, "ep-20240515062545-nnq8b")

        data = to_openai_completion_params(request)
        return self.client.chat.completions.create(**data)

    async def acreate(self, request: ChatCompletionRequest, **kwargs):
        request.model = self.endpoint_map.get(request.model, "ep-20240515062545-nnq8b")

        data = to_openai_completion_params(request)
        return await self.aclient.chat.completions.create(**data)

    @staticmethod
    # @ttl_cache(ttl=15 * 24 * 3600)
    def get_and_update_api_key(resource_ids):
        """pip install volcengine-python-sdk --no-deps -U"""
        import volcenginesdkcore
        import volcenginesdkark
        configuration = volcenginesdkcore.Configuration()
        configuration.ak = os.getenv("ARK_ACCESS_KEY")
        configuration.sk = os.getenv("ARK_SECRET_ACCESS_KEY")
        configuration.region = "cn-beijing"

        # set default configuration
        volcenginesdkcore.Configuration.set_default(configuration)

        # use global default configuration
        api_instance = volcenginesdkark.ARKApi()

        # for endpoint in endpoint_map.values():
        get_api_key_request = volcenginesdkark.GetApiKeyRequest(
            duration_seconds=30 * 24 * 3600,  # 有效期
            # resource_type="endpoint",
            resource_type="bot" if resource_ids[0].startswith("bot") else "endpoint",

            resource_ids=resource_ids
            # resource_ids=list(endpoint_map.values()),
            # resource_ids=["ep-20240515062545-nnq8b"]
        )

        resp = api_instance.get_api_key(get_api_key_request)

        return resp.api_key

    @property
    def endpoint_map(self):
        return {
            "doubao-lite-128k": "ep-20240520010805-566f7",
            "doubao-lite-32k": "ep-20240515062431-rkhfp",
            "doubao-lite-4k": "ep-20240515062545-nnq8b",

            "doubao-pro-128k": "ep-20240515073409-dlpqp",
            "doubao-pro-32k": "ep-20240515073434-s4skk",
            "doubao-pro-4k": "ep-20240515073524-xj4m2",  # fc

            "moonshot-v1-8k": "ep-20240516010405-mwthf",
            "moonshot-v1-32k": "ep-20240516010345-f88rs",
            "moonshot-v1-128k": "ep-20240516010323-zfzwj",

            "doubao-embedding": "ep-20240516005609-9c9pq",
            "name-20240606": "bot-20240605032421-b4j9j"
        }


if __name__ == '__main__':
    client = Completions()
    # print(client.get_and_update_api_key(resource_ids=list(client.endpoint_map.values())))
    print(client.get_and_update_api_key(resource_ids=["bot-20240605032421-b4j9j"]))

    # r = client.create(ChatCompletionRequest(stream=True, model="moonshot-v1-8k"))

    for i in r:
        print(i)



#!/usr/bin/env python
# -*- coding: utf-8 -*-
# @Project      : AI.  @by PyCharm
# @File         : completions
# @Time         : 2023/12/19 16:38
# @Author       : betterme
# @WeChat       : meutils
# @Software     : PyCharm
# @Description  : 逆向工程
import pandas as pd

from meutils.pipe import *
from meutils.notice.feishu import send_message
from meutils.serving.fastapi.dependencies.auth import get_bearer_token, HTTPAuthorizationCredentials

from fastapi import APIRouter, File, UploadFile, Query, Form, Depends, Request, HTTPException, status
from starlette.background import BackgroundTask, BackgroundTasks

from openai.types.chat import ChatCompletion, ChatCompletionChunk

from chatllm.completions.ali import Completions
from chatllm.schemas.openai_api_protocol import ChatCompletionRequest

router = APIRouter()

ChatCompletionResponse = Union[ChatCompletion, List[ChatCompletionChunk]]

from meutils.llm.openai_utils import create_chat_completion_chunk
from sse_starlette import EventSourceResponse, ServerSentEvent


@router.post("/chat/completions")
async def create_chat_completions(
    request: ChatCompletionRequest,
    auth: Optional[HTTPAuthorizationCredentials] = Depends(get_bearer_token),
    # background_tasks: BackgroundTasks = ...
):
    # background_tasks.add_task(send_message, "response")
    logger.debug(request)

    api_key = auth and auth.credentials or None

    response = await Completions(api_key=api_key).acreate(request)

    async def agen():  # 不正常
        for i in response:
            yield i.model_dump_json()
            await asyncio.sleep(0)

    # sse_response
    return EventSourceResponse(
        create_chat_completion_chunk(response),
        # agen(),
        # background=BackgroundTask(send_message, "response"),
        ping=10000
    )


if __name__ == '__main__':
    from meutils.serving.fastapi import App

    app = App()

    app.include_router(router, '/v1')

    app.run()
    # for i in range(10):
    #     send_message(f"兜底模型", title="github_copilot")



#!/usr/bin/env python
# -*- coding: utf-8 -*-
# @Project      : AI.  @by PyCharm
# @File         : completions
# @Time         : 2023/12/19 16:38
# @Author       : betterme
# @WeChat       : meutils
# @Software     : PyCharm
# @Description  : 逆向工程

from meutils.pipe import *
from meutils.notice.feishu import send_message
from meutils.serving.fastapi.dependencies.auth import get_bearer_token, HTTPAuthorizationCredentials

from sse_starlette import EventSourceResponse
from fastapi import APIRouter, File, UploadFile, Query, Form, Depends, Request, HTTPException, status

from openai import OpenAI, AsyncOpenAI
from openai.types.chat import ChatCompletion, ChatCompletionChunk

from chatllm.llmchain.completions.copilot import Completions

from chatllm.schemas.openai_api_protocol import ChatCompletionRequest, UsageInfo
from chatllm.utils.openai_utils import to_openai_completion_params, openai_response2sse

router = APIRouter()

ChatCompletionResponse = Union[ChatCompletion, List[ChatCompletionChunk]]

send_message = lru_cache(send_message)


@router.post("/chat/completions")
async def create_chat_completions(
    request: ChatCompletionRequest,
    auth: Optional[HTTPAuthorizationCredentials] = Depends(get_bearer_token),
):
    logger.debug(request)
    raw_model = request.model

    api_key = auth and auth.credentials or None
    if api_key is None:
        detail = {
            "error": {
                "message": "",
                "type": "invalid_request_error",
                "param": None,
                "code": "invalid_api_key",
            }
        }
        raise HTTPException(status_code=status.HTTP_401_UNAUTHORIZED, detail=detail)

    if "|" in api_key:
        api_key = np.random.choice(api_key.strip().split('|'))

    response = await Completions(api_key=api_key).acreate(request)
    return openai_response2sse(response, redirect_model=raw_model)


if __name__ == '__main__':
    from meutils.serving.fastapi import App

    app = App()

    app.include_router(router, '/v1')

    app.run()
    # for i in range(10):
    #     send_message(f"兜底模型", title="github_copilot")



#!/usr/bin/env python
# -*- coding: utf-8 -*-
# @Project      : AI.  @by PyCharm
# @File         : ali
# @Time         : 2024/5/24 11:11
# @Author       : betterme
# @WeChat       : meutils
# @Software     : PyCharm
# @Description  :
from asgiref.sync import sync_to_async, async_to_sync

from meutils.pipe import *
from meutils.io.image import base64_to_bytes, image_to_base64
from dashscope import Application, Generation, MultiModalConversation

from chatllm.schemas.openai_api_protocol import ChatCompletionRequest, UsageInfo
from chatllm.schemas.openai_types import chat_completion, chat_completion_chunk


class Completions(object):
    def __init__(self, api_key=None):
        self.api_key = api_key

    @sync_to_async
    def acreate(self, request: ChatCompletionRequest):
        return self.create(request)

    def create(self, request: ChatCompletionRequest):
        # {"text": null, "finish_reason": null, "choices": [
        #      {"finish_reason": "null", "message": {"role": "assistant", "content": [{"text": "这张"}]}}]},

        if request.model.__contains__("-vl"):  # 多模态
            # [{'role': 'user', 'content': [
            #   {"type": "text","text": "xx"},
            #   {"type": "image_url", "image_url": {"url": "xxx"}}]}
            # ]
            text = image_url = ""
            for content in request.messages[-1].get("content"):
                text = text or content.get("text")
                image_url = content.get("image_url", {}).get("url")

            # 兼容 base64
            if image_url.startswith("data:image"):
                from chatllm.llmchain.completions import chatglm_web
                upload = async_to_sync(chatglm_web.Completions().put_object_for_openai)

                file_object = upload(base64_to_bytes(image_url), purpose="file_upload")
                image_url = file_object.filename

                logger.debug(file_object)

            messages = [
                {
                    "role": "user",
                    "content": [
                        {"text": text},
                        {"image": image_url},
                    ]
                }
            ]
            logger.debug(messages)

            response = MultiModalConversation.call(
                model=request.model,  # app当做一个模型
                messages=messages,  # Message
                api_key=self.api_key,
                stream=request.stream,
                temperature=request.temperature,
            )


        elif request.model.startswith("qwen-") or request.model in {"farui-plus"}:
            response = Generation.call(
                model=request.model,  # app当做一个模型
                messages=request.messages,  # Message
                api_key=self.api_key,
                stream=request.stream,
                temperature=request.temperature,
                # incremental_output=True,
                # result_format='message',  # 设置输出为'message'格式
            )


        else:
            response = Application.call(
                app_id=request.model,  # app当做一个模型
                prompt=request.last_content,
                api_key=self.api_key,
                stream=request.stream,
                temperature=request.temperature,
            )

        if request.stream:
            content = ''
            for chunk in response:
                if chunk.status_code != 200:
                    logger.error(chunk)
                    chat_completion_chunk.choices[0].delta.content = chunk
                    chat_completion_chunk.choices[0].finish_reason = 'stop'
                    yield chat_completion_chunk
                    return

                output = chunk.get("output", {})

                _content = output.get("text")
                if _content:
                    chat_completion_chunk.choices[0].delta.content = _content.replace(content, '')
                    yield chat_completion_chunk
                    chat_completion_chunk.choices[0].finish_reason = output.get("finish_reason")
                    content = _content
                else:
                    _ = output.get("choices")[0]
                    finish_reason = _.get("finish_reason")
                    _content = _.get("message").get("content")
                    if isinstance(_content, list):
                        _content = _content[0].get("text")

                    chat_completion_chunk.choices[0].delta.content = _content.replace(content, '')
                    if finish_reason != 'stop':
                        chat_completion_chunk.choices[0].finish_reason = finish_reason
                    yield chat_completion_chunk
                    content = _content
        else:
            if response.status_code != 200:
                logger.error(response)
                chat_completion.choices[0].message.content = response
                yield chat_completion
                return

            chat_completion.choices[0].message.content = response.get("output", {}).get("text")

            if "models" in (usage := response.get("usage")):
                usage = response.get("usage").get("models")[0]

            prompt_tokens = usage.get("input_tokens")
            completion_tokens = usage.get("output_tokens")
            total_tokens = prompt_tokens + completion_tokens
            chat_completion.usage = UsageInfo(
                prompt_tokens=prompt_tokens,
                completion_tokens=completion_tokens,
                total_tokens=total_tokens
            )

            # logger.debug(chat_completion)

            yield chat_completion


if __name__ == '__main__':
    app_id = '3e27fc43adf5410c919cc4aaae03c88d'
    prompt = '你是谁'
    api_key = 'sk-4e6489ba597541dfa7a51f43e3912ca2'
    api_key = 'sk-9c519f1bffd4486f8c3c308ac3d89b66'

    model = "farui-plus"
    # model = "qwen-max"
    model = "qwen-vl-max"

    client = Completions(api_key=api_key)
    # print(client.get_and_update_api_key(resource_ids=list(client.endpoint_map.values())))

    # r = client.create(ChatCompletionRequest(stream=False, model=app_id))
    # r = client.create(ChatCompletionRequest(stream=False, model='qwen-max'))
    # r = client.create(
    #     ChatCompletionRequest(stream=True, model=model, messages=[{'role': 'user', 'content': '你是谁'}]))
    messages = [
        {'role': 'user', 'content': [
            {"type": "text", "text": "解释下"},
            # {"type": "image_url", "image_url": {"url": "https://oss.chatfire.cn/app/qun.png"}},
            # {"type": "image_url", "image_url": {"url": image_to_base64('x.png')}},

        ]
         }
    ]
    r = client.create(
        ChatCompletionRequest(stream=True, model=model, messages=messages)
    )

    for i in r:
        print(i)



#!/usr/bin/env python
# -*- coding: utf-8 -*-
# @Project      : AI.  @by PyCharm
# @File         : files
# @Time         : 2023/12/29 14:21
# @Author       : betterme
# @WeChat       : meutils
# @Software     : PyCharm
# @Description  : todo: 返回链接，参考kimi，接入文档解析，16c32g机器部署，不能通服务调用, minio_client通用化 id 映射

from meutils.pipe import *

from meutils.oss.minio_oss import Minio  # minio_client

from meutils.serving.fastapi.dependencies.auth import get_bearer_token, HTTPAuthorizationCredentials

from chatllm.utils.openai_utils import per_create
from chatllm.llmchain.document_loaders.file_loader import UnstructuredAPIFileLoader
from chatllm.llmchain.completions import chatglm_web, kimi

from enum import Enum
# from redis import Redis
from redis.asyncio import Redis

from openai import OpenAI, AsyncOpenAI
from openai._types import FileTypes
from openai.types.file_object import FileObject
from fastapi import APIRouter, File, UploadFile, Query, Form, BackgroundTasks, Depends, HTTPException, Request, status
from fastapi.responses import Response, FileResponse, JSONResponse

router = APIRouter()

# file_info = {}  # or Redis(**os.getenv("REDIS_CLIENT_PARAMS", {}), decode_responses=True)  # redis_client_params
file_info = Redis()  # redis_client_params

kimi_client = AsyncOpenAI(
    api_key=os.getenv('MOONSHOT_API_KEY'),
    base_url=os.getenv('MOONSHOT_BASE_URL'),
)


class Purpose(str, Enum):
    file_upload = "file-upload"
    file_upload_glm = "file-upload-glm"

    file_extract = "file-extract"  # kimi glm
    file_extract_kimi = "file-extract-kimi"

    file_extract_plus = "file-extract-plus"  # 自研

    rag = "rag"
    file_structuring = "file_structuring"
    file_embedding = "file_embedding"

    assistants = "assistants"
    fine_tune = "fine-tune"


OPENAI_BUCKET = os.getenv('OPENAI_BUCKET', 'test')

from fastapi import FastAPI
from fastapi.responses import RedirectResponse

app = FastAPI()


@app.get("/")
def go_to_baidu():
    return RedirectResponse("https://www.baidu.com")


@router.post("/files")  # 同名文件会被覆盖
async def upload_files(
    file: Union[UploadFile] = File(...),  # 文件链接
    purpose: Purpose = Form(...),
    auth: Optional[HTTPAuthorizationCredentials] = Depends(get_bearer_token),
):
    """
    todo: 存储redis
    """
    api_key = auth and auth.credentials or None
    if api_key is None:
        raise HTTPException(status_code=status.HTTP_401_UNAUTHORIZED, detail="认证失败")

    if purpose == Purpose.file_upload:  # 单纯的上传
        return await Minio().put_object_for_openai(file=file, purpose=purpose.value)

    # elif purpose == Purpose.file_extract_plus:
    #     content = await UnstructuredAPIFileLoader.load_for_openai(file)

    elif purpose == Purpose.file_extract:
        return await kimi_client.files.create(file=(file.filename, file.file), purpose=purpose.value)

    elif purpose == Purpose.file_extract_kimi:
        return kimi.Completions().file_extract(file=file)  # todo: 异步

    elif purpose == Purpose.file_upload_glm:
        return await chatglm_web.Completions().put_object_for_openai(file=file, purpose=purpose.value)

    elif purpose == Purpose.rag:
        raise HTTPException(status_code=status.HTTP_501_NOT_IMPLEMENTED, detail="暂不支持")


@router.get("/files/{file_id}/content")
async def get_files_content(
    file_id: str,
    auth: Optional[HTTPAuthorizationCredentials] = Depends(get_bearer_token),

):
    api_key = auth and auth.credentials or None
    if api_key is None:
        raise HTTPException(status_code=status.HTTP_401_UNAUTHORIZED, detail="认证失败")

    async with Redis(decode_responses=True) as redis_client:
        content = await redis_client.get(file_id)

        if file_id.startswith("cn"):
            if content is None:
                response = await kimi_client.files.content(file_id=file_id)
                await kimi_client.files.delete(file_id)  # 删除文件
                content = response.text

        if content is None:
            url = f"https://f.chatllm.vip/{OPENAI_BUCKET}/{file_id}"
            content = await UnstructuredAPIFileLoader.load_for_openai(url)
            content = json.dumps(content, ensure_ascii=False)

            # response = await Minio().aget_object(OPENAI_BUCKET, file_id)
            # response = Minio().get_object(OPENAI_BUCKET, file_id)
            # content = response.read()
            # file = io.BytesIO(content)

            # content = file_info.get(file_id)  # redis缓存

        # await redis_client.set(file_id, content)  # 为啥抛错
        redis_client.set(file_id, content)  # 为啥抛错

    # 解析内容不支持 files.content(file_id).stream_to_file('xx.pdf')
    return Response(content=content, media_type="application/json")  # json.dumps(content, ensure_ascii=False)


@router.get("/files/{file_id}")
async def get_files(
    file_id: str,
    auth: Optional[HTTPAuthorizationCredentials] = Depends(get_bearer_token),
):
    api_key = auth and auth.credentials or None
    if api_key is None:
        raise HTTPException(status_code=status.HTTP_401_UNAUTHORIZED, detail="认证失败")

    if file_id.startswith("cn"):  # kimi
        return await kimi_client.files.retrieve(file_id=file_id)

    # 映射 file_id
    # response = Minio().get_object(OPENAI_BUCKET, file_id)
    response = await Minio().aget_object(OPENAI_BUCKET, file_id)
    data = response.read()

    return FileObject(
        id=file_id,
        bytes=len(data),
        created_at=int(time.time()),
        filename=file_id,  # filename
        object='file',
        purpose='assistants',
        status='processed',
    )


@router.delete("/files/{file_id}")
async def delete_file(
    file_id: str,
    auth: Optional[HTTPAuthorizationCredentials] = Depends(get_bearer_token),
):
    api_key = auth and auth.credentials or None
    if api_key is None:
        raise HTTPException(status_code=status.HTTP_401_UNAUTHORIZED, detail="认证失败")

    if file_id.startswith("cn"):  # kimi
        response = await kimi_client.files.delete(file_id=file_id)
        return response


if __name__ == '__main__':
    from meutils.serving.fastapi import App

    VERSION_PREFIX = '/v1'

    app = App()
    app.include_router(router, VERSION_PREFIX)
    app.run(port=9000)



#!/usr/bin/env python
# -*- coding: utf-8 -*-
# @Project      : AI.  @by PyCharm
# @File         : suno_api
# @Time         : 2024/5/29 15:43
# @Author       : betterme
# @WeChat       : meutils
# @Software     : PyCharm
# @Description  :


from meutils.pipe import *
from meutils.llm.openai_utils import appu
from meutils.db.redis_db import redis_client
from meutils.notice.feishu import send_message
from meutils.serving.fastapi.dependencies.auth import get_bearer_token, HTTPAuthorizationCredentials

from chatllm.schemas.suno_types import SunoAIRequest
from chatllm.api.routers.suno.utils import API_GENERATE_V2, API_FEED, API_BILLING_INFO, api_generate_v2, api_feed, \
    api_billing_info

from fastapi import APIRouter, File, UploadFile, Query, Form, Header, Depends, Request, HTTPException, status
from fastapi.responses import JSONResponse

router = APIRouter()


@router.get(API_BILLING_INFO)
async def suno_api_billing_info(
    auth: Optional[HTTPAuthorizationCredentials] = Depends(get_bearer_token),
):
    api_key = auth and auth.credentials or None

    return api_billing_info(api_key)


@router.post(API_GENERATE_V2)
async def suno_api_generate_v2(
    request: SunoAIRequest,
    auth: Optional[HTTPAuthorizationCredentials] = Depends(get_bearer_token),
):
    logger.debug(request)

    api_key = auth and auth.credentials or None

    data = request.model_dump()

    return api_generate_v2(**data)


@router.get(API_FEED)
async def suno_api_feed(
    ids: str,
    auth: Optional[HTTPAuthorizationCredentials] = Depends(get_bearer_token),
):
    api_key = auth and auth.credentials or None

    return api_feed(api_key, ids)


if __name__ == '__main__':
    from meutils.serving.fastapi import App

    app = App()

    app.include_router(router, '/suno')

    app.run()



#!/usr/bin/env python
# -*- coding: utf-8 -*-
# @Project      : AI.  @by PyCharm
# @File         : free
# @Time         : 2024/4/25 11:18
# @Author       : betterme
# @WeChat       : meutils
# @Software     : PyCharm
# @Description  : https://free.p0.ee # 逆向画图吧

import httpx

from meutils.pipe import *
from meutils.notice.feishu import send_message
from meutils.str_utils.regular_expression import parse_url

from chatllm.schemas.openai_types import chat_completion, chat_completion_chunk
from chatllm.schemas.openai_api_protocol import ChatCompletionRequest, UsageInfo
from chatllm.utils.openai_utils import openai_response2sse, to_openai_completion_params

from openai import OpenAI, AsyncOpenAI

base_url = "https://free.p0.ee/api/auth"
model = "gpt-3.5-turbo"  # "@cf/bytedance/stable-diffusion-xl-lightning"


# Provider

class Completions(object):

    def __init__(self, ):
        self.base_url = "https://free.p0.ee/api/auth"

    def create(self, request: ChatCompletionRequest):
        provider = "/workers/image"
        if request.model.startswith(('gpt',)):
            provider = "/openai"
        elif request.model.startswith(('gemini',)):
            provider = '/gemini'

        with httpx.Client(base_url=self.base_url, timeout=20) as client:
            payload = {"endpoint": "/v1/chat/completions", **request.model_dump()}
            print(payload)

            response: httpx.Response
            with client.stream("POST", url=provider, json=payload) as response:

                for chunk in response.iter_lines():

                    print(chunk)

                # return response



if __name__ == '__main__':

    messages = [
        {
            "role": "user",
            "content": "hi"
        },
        {
            "role": "user",
            "content": "讲个故事"
        },

    ]
    request = ChatCompletionRequest(model='gpt-4', messages=messages)
    Completions().create(request)



#!/usr/bin/env python
# -*- coding: utf-8 -*-
# @Project      : AI.  @by PyCharm
# @File         : file_upload
# @Time         : 2024/5/31 15:25
# @Author       : betterme
# @WeChat       : meutils
# @Software     : PyCharm
# @Description  :

from meutils.pipe import *

from meutils.oss.minio_oss import Minio  # minio_client

from meutils.serving.fastapi.dependencies.auth import get_bearer_token, HTTPAuthorizationCredentials

from chatllm.utils.openai_utils import per_create
from chatllm.llmchain.document_loaders.file_loader import UnstructuredAPIFileLoader
from chatllm.llmchain.completions import chatglm_web, kimi

from enum import Enum
# from redis import Redis
from redis.asyncio import Redis

from openai import OpenAI, AsyncOpenAI
from openai._types import FileTypes
from openai.types.file_object import FileObject
from fastapi import APIRouter, File, UploadFile, Query, Form, BackgroundTasks, Depends, HTTPException, Request, status
from fastapi.responses import Response, FileResponse, JSONResponse

router = APIRouter()

from meutils.pipe import *

from meutils.oss.minio_oss import Minio  # minio_client

from meutils.serving.fastapi.dependencies.auth import get_bearer_token, HTTPAuthorizationCredentials

from chatllm.utils.openai_utils import per_create
from chatllm.llmchain.document_loaders.file_loader import UnstructuredAPIFileLoader
from chatllm.llmchain.completions import chatglm_web, kimi

from enum import Enum
# from redis import Redis
from redis.asyncio import Redis

from openai import OpenAI, AsyncOpenAI
from openai._types import FileTypes
from openai.types.file_object import FileObject
from fastapi import APIRouter, File, UploadFile, Query, Form, BackgroundTasks, Depends, HTTPException, Request, status
from fastapi.responses import Response, FileResponse, JSONResponse, RedirectResponse

router = APIRouter()


class Purpose(str, Enum):
    file_upload = "file-upload"
    file_upload_glm = "file-upload-glm"

    file_extract = "file-extract"  # kimi glm
    file_extract_kimi = "file-extract-kimi"

    file_extract_plus = "file-extract-plus"  # 自研

    rag = "rag"
    file_structuring = "file_structuring"
    file_embedding = "file_embedding"

    assistants = "assistants"
    fine_tune = "fine-tune"


@router.post("/files")  # 同名文件会被覆盖
async def upload_files(
    file: Union[UploadFile] = File(...),  # 文件链接
    purpose: Purpose = Form(...),
    auth: Optional[HTTPAuthorizationCredentials] = Depends(get_bearer_token),
):
    """
    todo: 存储redis
    """
    api_key = auth and auth.credentials or None
    if api_key is None:
        raise HTTPException(status_code=status.HTTP_401_UNAUTHORIZED, detail="认证失败")

    if purpose == Purpose.file_upload:  # 单纯的上传
        return await Minio().put_object_for_openai(file=file, purpose=purpose.value)

    # elif purpose == Purpose.file_extract_plus:
    #     content = await UnstructuredAPIFileLoader.load_for_openai(file)

    elif purpose == Purpose.file_extract:
        return await kimi_client.files.create(file=(file.filename, file.file), purpose=purpose.value)

    elif purpose == Purpose.file_extract_kimi:
        return kimi.Completions().file_extract(file=file)  # todo: 异步

    elif purpose == Purpose.file_upload_glm:
        return await chatglm_web.Completions().put_object_for_openai(file=file, purpose=purpose.value)

    elif purpose == Purpose.rag:
        raise HTTPException(status_code=status.HTTP_501_NOT_IMPLEMENTED, detail="暂不支持")


@router.post("/files")  # 同名文件会被覆盖
async def upload_files(
    file: Union[UploadFile] = File(...),  # 文件链接
    purpose: Purpose = Form(...),
    auth: Optional[HTTPAuthorizationCredentials] = Depends(get_bearer_token),
):
    """
    todo: 存储redis
    """
    api_key = auth and auth.credentials or None
    if api_key is None:
        raise HTTPException(status_code=status.HTTP_401_UNAUTHORIZED, detail="认证失败")

    if purpose == Purpose.file_upload:  # 单纯的上传
        return await Minio().put_object_for_openai(file=file, purpose=purpose.value)

    # elif purpose == Purpose.file_extract_plus:
    #     content = await UnstructuredAPIFileLoader.load_for_openai(file)

    elif purpose == Purpose.file_extract:
        return await kimi_client.files.create(file=(file.filename, file.file), purpose=purpose.value)

    elif purpose == Purpose.file_extract_kimi:
        return kimi.Completions().file_extract(file=file)  # todo: 异步

    elif purpose == Purpose.file_upload_glm:
        return await chatglm_web.Completions().put_object_for_openai(file=file, purpose=purpose.value)

    elif purpose == Purpose.rag:
        raise HTTPException(status_code=status.HTTP_501_NOT_IMPLEMENTED, detail="暂不支持")



import argparse
import contextlib
import json
import os
import re
import time
from http.cookies import SimpleCookie
from typing import Tuple
import random

from curl_cffi import requests
from curl_cffi.requests import Cookies
from fake_useragent import UserAgent
from requests import get as rget
from rich import print
from typing import Union

from dotenv import load_dotenv, find_dotenv

_ = load_dotenv(find_dotenv())

ua = UserAgent(browsers=["edge"])

get_session_url = "https://clerk.suno.ai/v1/client?_clerk_js_version=4.70.5"
exchange_token_url = (
    "https://clerk.suno.ai/v1/client/sessions/{sid}/tokens/api?_clerk_js_version=4.70.0"
)

base_url = "https://studio-api.suno.ai"
browser_version = "edge101"

HEADERS = {
    "Accept-Encoding": "gzip, deflate, br",
    "User-Agent": "Mozilla/5.0 (Windows NT 10.0; Win64; x64; rv:109.0) \
        Gecko/20100101 Firefox/117.0",
}

MUSIC_GENRE_LIST = [
    "African",
    "Asian",
    "South and southeast Asian",
    "Avant-garde",
    "Blues",
    "Caribbean and Caribbean-influenced",
    "Comedy",
    "Country",
    "Easy listening",
    "Electronic",
    "Folk",
    "Hip hop",
    "Jazz",
    "Latin",
    "Pop",
    "R&B and soul",
    "Rock",
]


class SongsGen:
    def __init__(self, cookie: str) -> None:
        self.session: requests.Session = requests.Session()
        HEADERS["user-agent"] = ua.random
        self.cookie = cookie
        self.session.cookies = self.parse_cookie_string(self.cookie)
        auth_token = self._get_auth_token()
        HEADERS["Authorization"] = f"Bearer {auth_token}"
        self.session.headers = HEADERS
        self.sid = None
        self.retry_time = 0
        # make the song_info_dict global since we can get the lyrics and song name first
        self.song_info_dict = {}
        # now data
        self.now_data = {}

    def _get_auth_token(self):
        response = self.session.get(get_session_url, impersonate=browser_version)
        data = response.json()
        r = data.get("response")
        sid = None
        if r:
            sid = r.get("last_active_session_id")
        if not sid:
            raise Exception("Failed to get session id")
        self.sid = sid
        response = self.session.post(
            exchange_token_url.format(sid=sid), impersonate=browser_version
        )
        data = response.json()
        return data.get("jwt")

    def _renew(self):
        response = self.session.post(
            exchange_token_url.format(sid=self.sid), impersonate=browser_version
        )
        self.session.headers["Authorization"] = f"Bearer {response.json().get('jwt')}"

    @staticmethod
    def parse_cookie_string(cookie_string):
        cookie = SimpleCookie()
        cookie.load(cookie_string)
        cookies_dict = {}
        for key, morsel in cookie.items():
            cookies_dict[key] = morsel.value
        return Cookies(cookies_dict)

    def get_limit_left(self) -> int:
        self.session.headers["user-agent"] = ua.random
        r = self.session.get(
            "https://studio-api.suno.ai/api/billing/info/", impersonate=browser_version
        )
        return int(r.json()["total_credits_left"] / 10)

    def _parse_lyrics(self, data: dict) -> Tuple[str, str]:
        song_name = data.get("title", "")
        mt = data.get("metadata")
        if (
            not mt
        ):  # Remove checking for title because custom songs have no title if not specified
            return "", ""
        lyrics = re.sub(r"\[.*?\]", "", mt.get("prompt"))
        return song_name, lyrics

    def _fetch_songs_metadata(self, ids):
        id1, id2 = ids[:2]
        url = f"https://studio-api.suno.ai/api/feed/?ids={id1}%2C{id2}"
        response = self.session.get(url, impersonate=browser_version)
        data = response.json()
        if type(data) == dict:
            if data.get("detail", "") == "Unauthorized":
                print("Token expired, renewing...")
                self.retry_time += 1
                if self.retry_time > 2:
                    song_name, lyric = self._parse_lyrics(self.now_data[0])
                    self.song_info_dict["song_name"] = song_name
                    self.song_info_dict["lyric"] = lyric
                    self.song_info_dict["song_url"] = (
                        f"https://audiopipe.suno.ai/?item_id={id1}"
                    )
                    print("will sleep 30 and try to download")
                    time.sleep(30)
                    # Done here
                    return True
                self._renew()
                time.sleep(5)
                return False
            else:
                data = [data]
        # renew now data
        self.now_data = data
        try:
            for d in data:
                # only get one url for now TODO: See if possible for both urls
                # and early return
                if audio_url := d.get("audio_url"):
                    song_name, lyric = self._parse_lyrics(d)
                    self.song_info_dict["song_name"] = song_name
                    self.song_info_dict["lyric"] = lyric
                    self.song_info_dict["song_url"] = audio_url
                    return True
            return False
        except Exception as e:
            print(e)
            # since we only get the music_id is ok
            # so we can make the id here and sleep some time
            print("Will sleep 45s and get the music url")
            time.sleep(45)
            song_name, lyric = self._parse_lyrics(self.now_data[0])
            self.song_info_dict["song_name"] = song_name
            self.song_info_dict["lyric"] = lyric
            self.song_info_dict["song_url"] = (
                f"https://audiopipe.suno.ai/?item_id={id1}"
            )
            # Done here
            return True

    def get_songs(
        self,
        prompt: str,
        tags: Union[str, None] = None,
        title: str = "",
        is_custom: bool = False,
    ) -> dict:
        url = f"{base_url}/api/generate/v2/"
        self.session.headers["user-agent"] = ua.random
        payload = {
            "gpt_description_prompt": prompt,
            "mv": "chirp-v3-0",
            "prompt": "",
            "make_instrumental": False,
        }
        if is_custom:
            payload["prompt"] = prompt
            payload["gpt_description_prompt"] = ""
            payload["title"] = title
            if not tags:
                payload["tags"] = random.choice(MUSIC_GENRE_LIST)
            else:
                payload["tags"] = tags
            print(payload)
        response = self.session.post(
            url,
            data=json.dumps(payload),
            impersonate=browser_version,
        )
        if not response.ok:
            print(response.text)
            raise Exception(f"Error response {str(response)}")
        response_body = response.json()
        songs_meta_info = response_body["clips"]
        request_ids = [i["id"] for i in songs_meta_info]
        start_wait = time.time()
        print("Waiting for results...")
        sleep_time = 10
        while True:
            if int(time.time() - start_wait) > 600:
                raise Exception("Request timeout")
            # TODOs support all mp3 here
            song_info = self._fetch_songs_metadata(request_ids)
            # spider rule
            if sleep_time > 2:
                time.sleep(sleep_time)
                sleep_time -= 2
            else:
                time.sleep(2)

            if not song_info:
                print(".", end="", flush=True)
            else:
                break
        # keep the song info dict as old api
        return self.song_info_dict

    def save_songs(
        self,
        prompt: str,
        output_dir: str = "./output",
        tags: Union[str, None] = None,
        title: Union[str, None] = None,
        is_custom: bool = False,
    ) -> None:
        mp3_index = 0
        try:
            self.get_songs(
                prompt, tags=tags, title=title, is_custom=is_custom
            )  # make the info dict
            song_name = self.song_info_dict["song_name"]
            lyric = self.song_info_dict["lyric"]
            link = self.song_info_dict["song_url"]
        except Exception as e:
            print(e)
            raise
        with contextlib.suppress(FileExistsError):
            os.mkdir(output_dir)
        print()
        while os.path.exists(os.path.join(output_dir, f"suno_{mp3_index}.mp3")):
            mp3_index += 1
        print(link)
        response = rget(link, allow_redirects=False, stream=True)
        if response.status_code != 200:
            raise Exception("Could not download song")
        # save response to file
        with open(
            os.path.join(output_dir, f"suno_{mp3_index + 1}.mp3"), "wb"
        ) as output_file:
            for chunk in response.iter_content(chunk_size=1024):
                # If the chunk is not empty, write it to the file.
                if chunk:
                    output_file.write(chunk)
        if not song_name:
            song_name = "Untitled"
        with open(
            os.path.join(output_dir, f"{song_name.replace(' ', '_')}.lrc"),
            "w",
            encoding="utf-8",
        ) as lyric_file:
            lyric_file.write(f"{song_name}\n\n{lyric}")


def main():
    parser = argparse.ArgumentParser()
    parser.add_argument("-U", help="Auth cookie from browser", type=str, default="")
    parser.add_argument(
        "--prompt",
        help="Prompt to generate songs for",
        type=str,
        required=True,
    )

    parser.add_argument(
        "--output-dir",
        help="Output directory",
        type=str,
        default="./output",
    )
    parser.add_argument(
        "--is_custom",
        dest="is_custom",
        action="store_true",
        help="use custom mode, need to provide title and tags",
    )
    parser.add_argument(
        "--title",
        help="Title of the song",
        type=str,
        default="",
    )
    parser.add_argument(
        "--tags",
        help="Tags of the song",
        type=str,
        default="",
    )

    args = parser.parse_args()

    # Create song generator
    # follow old style
    song_generator = SongsGen(
        os.environ.get("SUNO_COOKIE") or args.U,
    )
    print(f"{song_generator.get_limit_left()} times left")
    song_generator.save_songs(
        prompt=args.prompt,
        output_dir=args.output_dir,
        title=args.title,
        tags=args.tags,
        is_custom=args.is_custom,
    )


if __name__ == "__main__":
    main()



#!/usr/bin/env python
# -*- coding: utf-8 -*-
# @Project      : AI.  @by PyCharm
# @File         : vv
# @Time         : 2024/5/21 18:57
# @Author       : betterme
# @WeChat       : meutils
# @Software     : PyCharm
# @Description  : https://github.com/OpenBMB/MiniCPM-V?tab=readme-ov-file
# test.py    Need more than 16GB memory to run.
import torch
from PIL import Image
from transformers import AutoModel, AutoTokenizer

model = AutoModel.from_pretrained('openbmb/MiniCPM-Llama3-V-2_5-int4', trust_remote_code=True, low_cpu_mem_usage=True)
model = model.to(device='mps')

# tokenizer = AutoTokenizer.from_pretrained('openbmb/MiniCPM-Llama3-V-2_5', trust_remote_code=True)
# model.eval()
#
# image = Image.open('./assets/hk_OCR.jpg').convert('RGB')
# question = 'Where is this photo taken?'
# msgs = [{'role': 'user', 'content': question}]
#
# answer, context, _ = model.chat(
#     image=image,
#     msgs=msgs,
#     context=None,
#     tokenizer=tokenizer,
#     sampling=True
# )
# print(answer)



#!/usr/bin/env python
# -*- coding: utf-8 -*-
# @Project      : AI.  @by PyCharm
# @File         : gptall
# @Time         : 2024/3/13 14:06
# @Author       : betterme
# @WeChat       : meutils
# @Software     : PyCharm
# @Description  :

from meutils.pipe import *
from meutils.notice.feishu import send_message
from meutils.str_utils.regular_expression import parse_url

from chatllm.schemas.openai_types import chat_completion, chat_completion_chunk
from chatllm.schemas.openai_api_protocol import ChatCompletionRequest
from chatllm.utils.openai_utils import openai_response2sse, to_openai_completion_params

from openai import OpenAI, AsyncOpenAI


class Completions(object):

    def __init__(self, **client_params):
        self.api_key = client_params.get('api_key')
        self.client = AsyncOpenAI(
            api_key="sk-kVKomhMC01dfcz5N17A0E6D0FfB347CeB0323eEc143863Ae",
            base_url="https://api.zlrxjh.top/v1",
        )

        self.file_client = AsyncOpenAI(
            api_key=os.getenv('MOONSHOT_API_KEY'),
            base_url=os.getenv('MOONSHOT_BASE_URL'),
        )

    async def acreate(self, request: ChatCompletionRequest):
        """
        todo: 走低配版
        :param request:
        :return:
        """
        question = request.messages[-1]
        urls = parse_url(question.get('content', ''))
        for url in urls:
            async for chunk in self.file_extract(url):
                if isinstance(chunk, str):
                    request.messages.append({'role': 'user', 'content': chunk})  # pdf内容拼接在user里
                else:
                    yield chunk

        data = to_openai_completion_params(request, redirect_model='gpt-4-mobile')
        response = await self.client.chat.completions.create(**data)
        # async for chunk in response:
        #     yield chunk
        yield response

    def create_sse(self, request: ChatCompletionRequest):  # todo：非流式
        return openai_response2sse(self.acreate(request), redirect_model=request.model)

    async def file_extract(self, url: str):  # todo: 加缓存
        # urls = parse_url(content)

        async with httpx.AsyncClient(follow_redirects=True) as client:
            response = await client.get(url)
            if response.status_code != 200:
                chat_completion_chunk.choices[0].delta.content = f"\n> url error: {response.reason_phrase}\n"
                yield chat_completion_chunk
                return  # 应该是跳出

            file_object = await self.file_client.files.create(file=response.content, purpose="file-extract")

            # FileObject(id='cnoki7pkqq4r540f4q6g', bytes=154075, created_at=1710311711, filename='upload', object='file', purpose='file-extract', status='ok', status_details='')
            # status='error', status_details=''
            if file_object.status == 'ok':
                _ = file_object.model_dump_json()
                _ = f"\n```json\n{_}\n```\n"

                logger.debug(_)

                chat_completion_chunk.choices[0].delta.content = _
                yield chat_completion_chunk
            else:
                status_details = f"文件上传失败: {file_object.status_details}"
                send_message(status_details)
                chat_completion_chunk.choices[0].delta.content = f"\n```json\n{status_details}\n```\n"
                yield chat_completion_chunk
                return  # 应该是跳出

            # {
            #     "content": "2023年年度绩效面谈反馈表\n工号 7683 姓名 袁杰 岗位 金融算法工程师\n部门 信息科技部 中心 数据科学研发中心 上级 龚明升\n（1）做的好：\n 合规资讯全局和语义检索的实现，提高了搜索的准确性和效率。\n 智能文档解析服务项目的运行和文件服务器的建设，提高了文件处理的效率。\n 合规文档智能问答规丞相的实现，提供了一种基于大模型技术的新的用户交互方式。\n 爬虫需求的实现和优化，提高了数据获取的效率和准确性。\n 北斗客户旅程的客户分群模型的解释和优化，提高了模型的可解释性和准确性。\n（2）待优化：\n 合规宝典数据接入和更新的效率需要提高。\n 文档解析的准确率需要提高，目前的效果仍有待提升。\n 部分爬虫需求的实现存在问题，如部分网页附件下载失败，需要优化爬虫下载附件的逻辑。\n 部分合规资讯存在问题，如下载链接点击报错，文档中包含附件但无法下载等，需要进一步修复。\n 部分OCR解析需求的实现存在问题，需要进一步优化。\n 合规资讯bug的修复，提高了系统的稳定性。\n（3）24年重点工作：\n 继续优化合规宝典数据接入和更新的效率，目标是实现每日增量更新合规数据。\n 提高文档解析的准确率，包括常用类型的文档解析和特定类型（如财报）的文档解析。\n 优化爬虫需求的实现，解决部分网页附件下载失败等问题。\n 修复合规资讯存在的问题以及重构规丞相大模型智能问答，提高用户体验。\n 进一步优化OCR解析需求的实现，提高OCR识别的准确率。\n 进一步优化客户分群精准营销模型，提高模型的可解释性和准确性。\n 探索新的技术和方法，如Unstructured的可行性，大模型的升级和推理加速等。\n您在本周期的考评等级为 。\n员工对本次绩效考核结果的态度：\n 认同 □ 不认同 □ 无所谓\n员工签字\u0026日期：\n上级签字\u0026日期：\n\n说明：\n以上表格填写完成，由面谈双方签字后将原件或扫描件提交到部门负责人和 HRBP 处备案。",
            #     "file_type": "application/pdf", "filename": "upload", "title": "", "type": "file"
            # }

            yield await self.file_client.files.retrieve_content(file_id=file_object.id)



#!/usr/bin/env python
# -*- coding: utf-8 -*-
# @Project      : AI.  @by PyCharm
# @File         : suno
# @Time         : 2024/3/27 20:37
# @Author       : betterme
# @WeChat       : meutils
# @Software     : PyCharm
# @Description  : todo: # /api/session/ models


import jsonpath
from meutils.pipe import *
from meutils.schemas.suno_types import SunoAIRequest, BASE_URL, API_GENERATE_V2, API_FEED, API_BILLING_INFO, API_SESSION


def get_refresh_token(api_key):  # 定时更新一次就行
    """
    last_active_session_id, last_active_token
    """
    headers = {
        "User-Agent": "Mozilla/5.0 (Windows NT 10.0; Win64; x64) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/117.0.0.0 Safari/537.36",
        "Cookie": f"__client={api_key}"
    }
    # url = f"https://clerk.suno.com/v1/client?_clerk_js_version={_CLERK_JS_VERSION}"
    url = f"https://clerk.suno.com/v1/client"

    response = httpx.get(url, headers=headers)
    logger.debug(response.text)
    if response.status_code == 200:
        obj = response.json()
        # last_active_token = jsonpath.jsonpath(obj, "$..jwt")[0]
        last_active_session_id = jsonpath.jsonpath(obj, "$..last_active_session_id")[0]

        return last_active_session_id

    return response.text


@lru_cache()  # 30s
def get_access_token(api_key):
    last_active_session_id = get_refresh_token(api_key)  # last_active_token 没啥用

    # url = f"https://clerk.suno.com/v1/client/sessions/{last_active_session_id}/tokens?_clerk_js_version={_CLERK_JS_VERSION}"
    url = f"https://clerk.suno.com/v1/client/sessions/{last_active_session_id}/tokens"

    headers = {
        # "Authorization": f"Bearer {last_active_token}",
        "Cookie": f"__client={api_key}"
    }

    response = httpx.post(url, headers=headers)
    if response.status_code == 200:
        return response.json().get('jwt')
    return response.text


def api_generate_v2(api_key, payload):
    access_token = get_access_token(api_key)
    headers = {
        "Authorization": f"Bearer {access_token}",
    }
    httpx_client = httpx.Client(base_url=BASE_URL, headers=headers)

    response = httpx_client.post(API_GENERATE_V2, json=payload)
    if response.status_code == 200:
        return response.json()  # jsonpath.jsonpath(dd, "$.clips..id")
    return response.text


def api_feed(api_key, ids):
    access_token = get_access_token(api_key)
    headers = {
        "Authorization": f"Bearer {access_token}",
    }
    httpx_client = httpx.Client(base_url=BASE_URL, headers=headers)

    response = httpx_client.get(API_FEED, params={"ids": ids})
    if response.status_code == 200:
        return response.json()  # jsonpath.jsonpath(dd, "$.clips..id")
    return response.text


def api_billing_info(api_key):
    access_token = get_access_token(api_key)
    logger.debug(access_token)

    headers = {
        "Authorization": f"Bearer {access_token}",
    }
    httpx_client = httpx.Client(base_url=BASE_URL, headers=headers)

    response = httpx_client.get(API_BILLING_INFO)
    if response.status_code == 200:
        return response.json()  # total_credits_left
    return response.text


def api_session(api_key):
    access_token = get_access_token(api_key)

    headers = {
        "Authorization": f"Bearer {access_token}",
    }
    httpx_client = httpx.Client(base_url=BASE_URL, headers=headers)

    response = httpx_client.get(API_SESSION)
    if response.status_code == 200:
        return response.json()['models']  # total_credits_left
    return response.text


if __name__ == '__main__':
    api_key = os.getenv("SUNO_API_KEY")
    print(get_refresh_token(api_key))
    # print(get_access_token(api_key))
    # payload = {
    #     "gpt_description_prompt": "a romantic ballad song about not being able to wait to see you again",
    #     "mv": "chirp-v3-5",
    #     "prompt": "",
    #     "make_instrumental": True
    # }
    # payload = {
    #     "prompt": "[Verse]\nSun is high and the skies are blue\nFeels so good just being with you\n\n[Verse 2]\nWe got time and a world to see\nLiving life and we're feeling free\n\n[Chorus]\nOh oh sunny day delight\nEverything just feels so right\nOh oh shining so bright\nDance with me till the night\n\n[Verse 3]\nRunning wild under golden skies\nSmile so big can't believe my eyes\n\n[Bridge]\nHand in hand we can touch the stars\nDoesn't matter how far we are\n\n[Chorus]\nOh oh sunny day delight\nEverything just feels so right\nOh oh shining so bright\nDance with me till the night",
    #     "tags": "r&b",
    #     "mv": "chirp-v3-5",
    #     "title": "Sunny Day Delight",
    #     "continue_clip_id": None,
    #     "continue_at": None,
    #     "infill_start_s": None,
    #     "infill_end_s": None
    # }

    payload = {
        # 优先级最高
        "prompt": "[Verse]\nSun is high and the skies are blue\nFeels so good just being with you\n\n[Verse 2]\nWe got time and a world to see\nLiving life and we're feeling free\n\n[Chorus]\nOh oh sunny day delight\nEverything just feels so right\nOh oh shining so bright\nDance with me till the night\n\n[Verse 3]\nRunning wild under golden skies\nSmile so big can't believe my eyes\n\n[Bridge]\nHand in hand we can touch the stars\nDoesn't matter how far we are\n\n[Chorus]\nOh oh sunny day delight\nEverything just feels so right\nOh oh shining so bright\nDance with me till the night",
        # "prompt": "",

        # "gpt_description_prompt": "创作一首中文歌曲",

        "mv": "chirp-v3-5",

        "make_instrumental": True,
        "tags": "r&b",
        "title": "Chatfire",
        "continue_clip_id": None,
        "continue_at": None,
        "infill_start_s": None,
        "infill_end_s": None
    }

    # print(api_generate_v2(api_key, payload))
    # ids = "ee6d4369-3c75-4526-b6f1-b5f2f271cf30"
    # print(api_feed(api_key, ids))

    for i in range(100):  # 测试过期时间
        print(api_session(api_key))
        time.sleep(10)
